//= require <dSS/util/Util>

Ext.namespace('dSS', 'dSS.grid');

dSS.grid.DeviceCacheDetails = Ext.extend(Ext.grid.GridPanel,
{
    initComponent: function()
    {
        var cacheReadoutRenderer = function(value, metaData, record, rowIndex, colIndex, store) {
            return value === "" ? "" : String(Math.round(value, 10)) + "%";
        };

        var columns =
        [
            {
                header: dSS._("Room name"),
                tooltip: dSS._("Room name"),
                width: 80,
                sortable: true,
                dataIndex: 'zoneName',
                renderer: dSS.util.gridZoneNameRenderer
            },
            {
                header: "",
                width: 20,
                resizable: false,
                sortable: false,
                hideable: false,
                menuDisabled: true,
                fixed: true,
                dataIndex: 'functionID',
                renderer: dSS.util.gridDeviceTypeRenderer
            },
            {
                header: dSS._("Name"),
                tooltip: dSS._("Name"),
                width: 80,
                sortable: true,
                dataIndex: 'name',
                    renderer: dSS.util.gridDeviceNameRenderer
            },
            {
                header: dSS._("ID"),
                tooltip: dSS._("ID"),
                width: 70,
                sortable: true,
                    dataIndex: 'dsuid',
                renderer: dSS.util.gridDeviceDSUIDRenderer
            },
            {
                header: dSS._("Cache readout"),
                tooltip: dSS._("Cache readout"),
                width: 70,
                sortable: true,
                dataIndex: 'cacheReadout',
                renderer: cacheReadoutRenderer
            }
        ];

        Ext.apply(this, {
            store: dSS.data.Loader.getDeviceStore(),
            columns: columns,
            stripeRows: true,
            disableSelection: true,
            viewConfig: {
                forceFit: true
            }
        });

        dSS.grid.DeviceCacheDetails.superclass.initComponent.apply(this, arguments);
        this.on('afterrender', function(me) {
            me.sortState = this.getStore().getSortState();
            me.getStore().clearFilter();
            me.getStore().filterBy(function(record) {
                return !isNaN(parseInt(record.get('cacheReadout'), 10));
            }, this);
            me.getStore().sort('zoneName', 'ASC');
        }, this);

        this.on('beforedestroy', function(me) {
            if (me.sortState !== undefined) {
                me.getStore().sort(me.sortState.field,
                                   me.sortState.direction);
            }
        }, this);
    }
});

Ext.reg('dssdevicecachedetails', dSS.grid.DeviceCacheDetails);

dSS.DeviceCacheInfo = Ext.extend(Ext.form.FormPanel,
{
    initComponent: function()
    {
        Ext.apply(this, {
            layout: 'form',
            bodyBorder: false,
            items:
            [
                {
                    xtype: 'fieldset',
                    title: dSS._("Device Cache"),
                    layout: 'form',
                    scope: this,
                    ref: 'mainSet',
                    items:
                    [
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            html: dSS._("The system caches device data to " +
                                        "speed up device configuration and " +
                                        "to determine room, group and device " +
                                        "states. The states may differ from " +
                                        "the real states until finishing " +
                                        "readout. Automatic cache update is " +
                                        "performed periodically and on " +
                                        "changes.")
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            height: 25,
                            ref: 'deviceLabel'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            ref: 'statusLabel',
                            html: dSS._("Readout status") + ":"
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'progress',
                            ref: 'progress'
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            html: dSS._("Remaining readout time depends on " +
                                        "system usage.")
                        },
                        {
                            xtype: 'box',
                            height: 10
                        },
                        {
                            xtype: 'label',
                            cls: 'x-green-text',
                            ref: 'estimationLabel'
                        }
                    ],
                    buttonAlign: 'left',
                    buttons:
                    [
                        {
                            text: dSS._("Details"),
                            handler: this.showDetails.createDelegate(this)
                        },
                        '->',
                        {
                            text: dSS._("Update Cache"),
                            handler: this.updateCache.createDelegate(this)
                        }
                    ]
                }
            ]
        });

        dSS.DeviceCacheInfo.superclass.initComponent.apply(this, arguments);

        var unmask = dSS.util.maskViewport.createDelegate(this, [ 'unmask' ]);
        var mask = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);

        dSS.MessageBus.on('reloadapartment', function() {
            if (!this.isVisible()) {
                return;
            }
            mask();
        }, this);

        dSS.MessageBus.on('structureloaded', function() {
            if (!this.isVisible()) {
                return;
            }
            unmask();
            this.getCacheStatus();
        }, this);

        dSS.MessageBus.on('refreshview', this.reloadData.createDelegate(this),
            this);
    },
    reloadData: function() {
        if (dSS.config.firstLoad === true) {
            var fn = dSS.util.maskViewport.createDelegate(this, [ 'mask' ]);
            fn();
            dSS.data.Loader.load();
            dSS.config.firstLoad = false;
         } else {
            this.getCacheStatus();
        }
    },
    getCacheStatus : function() {
        if (!this.isVisible()) {
            return;
        }

        Ext.Ajax.request({
            url: '/api/v1/installation/parameterCache',
            disableCaching: true,
            timeout: 60000,
            method: 'GET',
            scope: this,
            success: function(response) {
                try {
                    if (response.status >= 300) {
                        Ext.Msg.alert(_("Error"),
                            dSS._("Could not determine device cache status"));
                        return;
                    }


                    var json = Ext.util.JSON.decode(response.responseText);
                    readyMsg = dSS.gt.strargs(
                                    dSS._("Cache ready for <b>%1</b> of " +
                                          "<b>%2</b> devices."),
                                    [
                                        json.data.attributes.completed,
                                        json.data.attributes.total
                                    ]);

                    var updaterStatus = dSS._("Readout status") + ": "
                    if (json.data.attributes.status === "running") {
                        updaterStatus += dSS._("running");
                    } else {
                        updaterStatus += dSS._("stopped");
                    }
                    this.mainSet.statusLabel.update(updaterStatus);

                    var secs = parseInt(json.data.attributes.remainingTime, 10);
                    var hours = Math.floor(secs / 3600);
                    var minutes = Math.ceil(secs / 60) % 60;

                    var remaining = "";
                    if (hours > 0) {
                        remaining = String(hours) + dSS._("h") + " ";
                    }
                    if (minutes > 0) {
                        remaining += String(minutes) + dSS._("m");
                    }

                    this.mainSet.deviceLabel.update(readyMsg);
                    this.mainSet.progress.updateProgress(
                            json.data.attributes.completion / 100.0,
                            String(Math.floor(json.data.attributes.completion, 10)) + "%");
                    if (remaining.length === 0) {
                        this.mainSet.estimationLabel.update("");
                    } else {
                        this.mainSet.estimationLabel.update(dSS._("Current estimation") + ":&nbsp;<b>" + remaining + "</b>");
                    }

                    var i;
                    var store = dSS.data.Loader.getDeviceStore();

                    for (i = 0; i < json.data.included.functionBlocks.length; i++) {
                        var entry = json.data.included.functionBlocks[i];
                        var record = store.getById(entry.id);
                        if ((record !== undefined) &&
                            (entry.attributes.completion !== undefined)) {
                            record.set('cacheReadout', entry.attributes.completion);
                            record.commit();
                        }
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                            dSS._("Could not determine device cache status") +
                            ": " + dSS._(error.message));
                }
            },
            failure: function() {
                 Ext.Msg.alert(_("Error"),
                           dSS._("Could not determine device cache status"));
            }
        });
    },
    updateCache: function() {
        Ext.Ajax.request({
            url: '/api/v1/installation/parameterCache/update',
            disableCaching: true,
            jsonData: Ext.util.JSON.encode({ mode: 'slow' }),
            scope: this,
            success: function(response) {
                try {
                    if (response.status  >= 300) {
                        Ext.Msg.alert(_("Error"),
                            dSS._("Could not trigger device cache update"));
                        return;
                    }
                } catch (error) {
                    Ext.Msg.alert(_("Error"),
                            dSS._("Could not trigger device cache update") +
                            ": " + dSS._(error.message));
                }
            },
            failure: function() {
                Ext.Msg.alert(_("Error"),
                        dSS._("Could not trigger device cache update"));
            }
        });
    },
    showDetails: function() {
        var details = new Ext.Window({
            title: dSS._("Device Cache Details"),
            width: 600,
            height: 400,
            modal: true,
            layout: 'fit',
            items: [ { xtype: 'dssdevicecachedetails' } ],
            buttons:
            [
                {
                    text: dSS._("Close"),
                    handler: function() { details.close(); }
                }
            ]
        });
        details.show();
    }
});

Ext.reg('dssdevicecacheinfo', dSS.DeviceCacheInfo);

