Ext.namespace("dSS", "dSS.devices", "dSS.config", "dSS.gthelper", "Ext.ux");

dSS.devices.DeviceSettingsVdcConfigLinkView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.dSUID = config.dSUID;
        this.deviceRecord = config.device;

        Ext.apply(this, {
            padding: 5,
            layout: 'absolute',
            border: false,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged', 
                            'enableme' ],
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    padding: 5,
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Clicking the button will open the vDC " +
                                        "configuration page in a new tab or window."),
                            x: 0,
                            y: 30
                        }
                    ]
                },
                {
                    xtype: 'button',
                    x: ((dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 34) / 2) - (260 / 2),
                    y: (config.initialHeight / 2) - (25 / 2),
                    minWidth: 260,
                    height: 25,
                    handler: this.openVdcConfigPage.createDelegate(this),
                text: dSS._("Open vDC configuration")
                }
            ]
        });

        dSS.devices.DeviceSettingsVdcConfigLinkView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);
    },
    openVdcConfigPage: function(record) {
        if (this.deviceRecord) {
            dSS.util.openVdcLink(this.deviceRecord, 'standardaction');
        }
    },
    killRequests: function() {
        if (this.AjaxRequestConfigLink !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestConfigLink);
            this.AjaxRequestConfigLink = undefined;
        }
    },
    loadData: function(reqdSUID, record, shownow) {
        this.killRequests();

        this.dSUID = reqdSUID;

        this.deviceRecord = record;

        this.fireEvent('loading', true);

        this.AjaxRequestConfigLink = Ext.Ajax.request({
            url: '/json/property/query',
            method: 'GET',
            params:
            {
                query: '/apartment/zones/zone0/devices/' + this.dSUID +
                       '/productInfo(ConfigLink,State,ProductState)'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestConfigLink = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if ((jsonData.ok !== true) ||
                        (jsonData.result == undefined) ||
                        (jsonData.result.productInfo == undefined) ||
                        (jsonData.result.productInfo.length == 0) ||
                        (jsonData.result.productInfo[0].State !== "Valid") ||
                        (jsonData.result.productInfo[0].ProductState !== "Valid")) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("The configuration page location is " +
                                  "unknown and cannot be loaded."));
                        return;
                    } else {
                        // this dialog is an exception in relation to
                        // the enableme event; the user can not change anything
                        // here, only clicking the button is allowed, so
                        // the dialog will stay disabled while loading
                        this.fireEvent('enableme');
                        if (shownow === true) {
                            this.openVdcConfigPage();
                        }
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("The configuration page location is " +
                              "unknown and cannot be loaded."));
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestConfigLink = undefined;
                if (this.closed === true) {
                    return;
                }
            }
        });
    }
});

dSS.devices.DeviceSettingsHSVView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.closed = false;

        this.AjaxRequestOutputChannels = undefined;

        this.previousColor = { h: -1, s: -1, v: -1 };
        this.defaultColor = { h: 0, s: 0, v: 0 };
        this.dirtyColor = false;
        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.dSUID = config.dSUID;

        var localizePicker =
        {
            text:
            {
                title: dSS._("Drag markers to pick a color")
            },
            tooltips:
            {
                hue:
                {
                    radio: dSS._("Set to Hue color mode"),
                    textbox: dSS._("Enter a Hue value (0-360°)")
                },
                saturation:
                {
                    radio: dSS._("Set to Saturation color mode"),
                    textbox: dSS._("Enter a Saturation value (0-100%)")
                },
                value:
                {
                    radio: dSS._("Set to Value color mode"),
                    textbox: dSS._("Enter a Value value (0-100%)")
                }
            }
        };

        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            border: false,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme' ],
            items:
            [
                {
                    xtype: 'jpickerpanel',
                    ref: 'colorPicker',
                    border: false,
                    localization: localizePicker
                }
            ]
        });

        dSS.devices.DeviceSettingsHSVView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.colorPicker.on('jpickercolorchanged',
                this.handleColorChanged.createDelegate(this), this);

    },
    setColorOnDevice: function(color) {
        // hue usually goes from 0 to 360°
        if (color.h > 360) {
            color.h = 360;
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: 'hue=' + color.h +
                               ';saturation=' + color.s +
                               ';brightness=' + color.v,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set device color")
        });
    },

    handleColorChanged: function(panel, color) {
        this.killRequests();

        this.fireEvent('formchanged');

        if (this.apartmentMode === true) {
            return;
        }

        if (this.colorPicker.equalHSVColors(this.previousColor, color)) {
            return;
        }

        this.setColorOnDevice(color);
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }
        var color = this.colorPicker.getColor();
        this.setColorOnDevice(color);
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }

        if (this.apartmentMode !== true) {
            return;
        }
        this.killRequests();

        var color = this.colorPicker.getColor();
        if (color.h > 360) {
            color.h = 360;
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelSceneValue',
            scope: this,
            params:
            {
                channelvalues: 'hue=' + color.h +
                               ';saturation=' + color.s +
                               ';brightness=' + color.v,
                dsuid: this.dSUID,
                sceneNumber: scene
            },
            failureMsg: dSS._("Could not set device color")
        });
        this.resetDefaults();
    },
    clearData: function() {
        this.killRequests();
        this.colorPicker.setColor(this.defaultColor);
    },
    killRequests: function() {
        this.dirtyColor = true;
        this.fireEvent('loading', false);
        if (this.AjaxRequestOutputChannels !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestOutputChannels);
            this.AjaxRequestOutputChannels = undefined;
        }
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }

        this.previousColor = this.colorPicker.getColor();
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }
        this.colorPicker.setColor(this.previousColor);
    },
    loadData: function(reqdSUID) {
        this.killRequests();

        this.dSUID = reqdSUID;

        this.dirtyColor = false;
        this.fireEvent('loading', true);

        this.fireEvent('enableme');

        this.AjaxRequestOutputChannels = Ext.Ajax.request({
            url: '/json/device/getOutputChannelValue',
            method: 'GET',
            params:
            {
                dsuid: reqdSUID,
                channels: 'hue;saturation;brightness'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOutputChannels = undefined;
                if ((this.closed === true) ||
                    (this.dirtyColor === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }
                        
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current output channel " +
                                  "values") + ': ' + jsonData.message);
                        return;
                    }

                    var hue;
                    var sat;
                    var val;
                    var i;
                    for (i = 0; i < jsonData.result.channels.length; i++) {
                        var chan = jsonData.result.channels[i];
                        if (chan.channel === "hue") {
                            hue = Math.round(chan.value * 10) / 10;
                            continue;
                        }
                        if (chan.channel === "saturation") {
                            sat = Math.round(chan.value * 10) / 10;
                            continue;
                        }
                        if (chan.channel === "brightness") {
                            val = Math.round(chan.value * 10) / 10;
                            continue;
                        }
                    }

                    if (!isNaN(hue) && !isNaN(sat) && !isNaN(val)) {
                        this.previousColor.h = hue;
                        this.previousColor.s = sat;
                        this.previousColor.v = val;

                        this.colorPicker.setColor(this.previousColor);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOutputChannels = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            }
        });
    },
    setColor: function(color) {
        this.previousColor = color;
        this.colorPicker.setColor(color);
    }
});

dSS.devices.DeviceSettingsDefaultView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var xPos = config.xPos;
        var xPosOffset = config.xPosOffset;

        this.maxValue = config.maxValue;
        this.closed = false;
        this.dSUID = config.dSUID;

        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.dirtyValue = false;
        this.dirtyAngle = false;

        this.previousValue = 0;
        this.previousBladeAngle = 0;
        this.decPrecision = (config.decPrecision === 0) ? 1 : 0.001;

        var increment = this.decPrecision;
        if ((this.apartmentMode === true) &&
            (config.matrix.grkl387workaround === true)) {
            increment = dSS.config.GRKL387SliderIncrement;
        }

        Ext.apply(this, {
            layout: 'absolute',
            border: false,
            padding: 5,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme' ],
            items:
            [
                {
                    xtype: 'label',
                    ref: 'valueLabel',
                    x: 10,
                    y: 16,
                    disabled: this.dummy,
                    text: dSS._("Value") + ':'
                },
                {
                    xtype: 'numberfield',
                    ref: 'valueField',
                    x: xPos - xPosOffset,
                    y: 12,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: (config.decPrecision !== 0),
                    decimalPrecision: config.decPrecision,
                    forceDecimalPrecision: true,
                    disabled: (config.devClass === dSS.config.deviceClass.BL),
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleValueChanged.createDelegate(this)
                    }
                },
                {
                    x: xPos - (xPosOffset - 55),
                    y: 16,
                    xtype: 'label',
                    text: '%'
                },
                {
                    xtype: 'slider',
                    ref: 'valueSlider',
                    x: xPos - (xPosOffset - 70),
                    y: 12,
                    height: 150,
					value: 0,
                    minValue: 0,
                    maxValue: 100,
                    vertical: true,
                    increment: increment,
                    decimalPrecision: config.decPrecision,
                    disabled: (config.devClass === dSS.config.deviceClass.BL),
                    plugins: new Ext.slider.Tip({
                        getText: function(thumb) {
                            return String.format('{0}%', thumb.value);
                        }
                    }),
                    listeners: {
                        changeComplete: this.handleValueChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'button',
                    ref: 'stopButton',
                    x: xPos - xPosOffset,
                    y: 140,
                    text: dSS._("Stop"),
                    width: 50,
                    handler: this.handleStop.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'maxButton',
                    x: xPos - (xPosOffset - 102),
                    y: 12,
                    text: dSS._("Max"),
                    tooltip: dSS._("Set maximum value"),
                    width: 130,
                    handler: this.handleMax.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'minButton',
                    x: xPos - (xPosOffset - 102),
                    y: 140,
                    text: dSS._("Min"),
                    tooltip: dSS._("Set minimum value"),
                    width: 130,
                    handler: this.handleMin.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'incButton',
                    x: xPos - (xPosOffset - 102),
                    y: 54,
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._("Increase value"),
                    width: 130,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'decButton',
                    x: xPos - (xPosOffset - 102),
                    y: 98,
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._("Decrease value"),
                    width: 130,
                    handler: this.handleDec.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'forcedModeOnButton',
                    x: xPos - (xPosOffset - 102),
                    y: 12,
                    text: dSS._("Forced mode on"),
                    tooltip: dSS._("Forced mode on"),
                    width: 130,
                    hidden: (config.devClass !== dSS.config.deviceClass.BL),
                    handler: this.handleForcedModeOn.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'forcedModeOffButton',
                    x: xPos - (xPosOffset - 102),
                    y: 76,
                    text: dSS._("Forced mode off"),
                    tooltip: dSS._("Forced mode off"),
                    width: 130,
                    hidden: (config.devClass !== dSS.config.deviceClass.BL),
                    handler: this.handleForcedModeOff.createDelegate(this)
                },
                {
                    xtype: 'button',
                    ref: 'noForcedModeButton',
                    x: xPos - (xPosOffset - 102),
                    y: 140,
                    text: dSS._("Normal mode"),
                    tooltip: dSS._("Normal mode"),
                    width: 130,
                    hidden: (config.devClass !== dSS.config.deviceClass.BL),
                    handler: this.handleNoForcedMode.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'opModeLabel',
                    x: 10,
                    y: 125,
                    text: dSS._("Operation mode") + ':',
                    hidden: (config.devClass !== dSS.config.deviceClass.BL)
                },
                {
                    xtype: 'label',
                    ref: 'opModeText',
                    x: 10,
                    y: 145,
                    hidden: (config.devClass !== dSS.config.deviceClass.BL)
                },
                {
                    xtype: 'label',
                    ref: 'bladeAngleLabel',
                    x: 10,
                    y: 178,
                    text: dSS._("Set blade angle") + ':'
                },
                {
                    xtype: 'numberfield',
                    ref: 'bladeAngleField',
                    x: xPos - xPosOffset,
                    y: 174,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleBladeAngleChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'label',
                    ref: 'bladeAngleUnit',
                    x: xPos - (xPosOffset - 55),
                    y: 178,
                    text: '%'
                },
                {
                    xtype: 'slider',
                    ref: 'angleSlider',
                    x: xPos - (xPosOffset - 74),
                    y: 174,
                    width: 160,
		            value: 0, 
                    increment: 1,
                    minValue: 0,
                    maxValue: 100,
                    vertical: false,
                    decimalPrecision: 1,
                    plugins: new Ext.slider.Tip({
                        getText: function(thumb) {
                            return String.format('{0}%', thumb.value);
                        }
                    }),
                    listeners: {
                        changeComplete: this.handleBladeAngleChanged.createDelegate(this)
                    }
                },
                {
                    xtype: 'label',
                    ref: 'grKLHack',
                    x: 10,
                    y: 205,
                    text: dSS._("This device only supports output values in 6.66% steps."),
                    hidden: !((this.apartmentMode === true) && (config.matrix.grkl387workaround === true))
                }
            ]
        });

        dSS.devices.DeviceSettingsDefaultView.superclass.constructor.apply(this, arguments);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.valueField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = parseFloat(field.getValue());
                    var ov = parseFloat(this.valueSlider.getValue());
                    if ((this.apartmentMode === true) &&
                        (this.matrix.grkl387workaround === true)) {
                        nv = (Math.round(nv*15/100)/15)*100;
                        field.setValue(nv);
                    };
                    if (nv !== ov) {
                        this.handleValueChanged(field, nv);
                    }
                }
            },
            this);

        this.bladeAngleField.on('specialkey', function(field, e) {
                if (e.getKey() == e.ENTER) {
                    var nv = field.getValue();
                    this.handleBladeAngleChanged(field, nv);
                }
            },
            this);
    },
    setMatrix: function(matrix) {
        this.matrix = matrix;
    },
    handleValueChanged: function(container, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.valueSlider.setValue(newVal);
        this.valueField.setValue(newVal);
        this.dirtyValue = true;
        if (this.apartmentMode !== true) {
            this.setValueOnDevice(newVal);
        }
        if (this.matrix.shadebladeang === true) {
            if (newVal >= 90) {
                this.handleBladeAngleChanged.defer(300, this,
                        [ this.bladeAngleField, 100 ]);
            }
        }
    },
    setValueOnDevice: function(value) {
        this.previousValue = value;

        if (this.maxValue <= 0xff) {
            dSS.util.setValueOnDevice(this.dSUID,
                                      dSS.util.percentToValue(value,
                                                    this.maxValue));
        } else {
            dSS.util.setOutputValueOnDevice(this.dSUID, 0x02,
                                        dSS.util.percentToValue(value,
                                                    this.maxValue));
        }
    },
    handleBladeAngleChanged: function(container, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.dirtyAngle = true;
        this.bladeAngleField.setValue(newVal);
        this.angleSlider.setValue(newVal);

        if ((this.apartmentMode !== true) &&
            (newVal !== this.previousBladeAngle)) {
            this.setAngleOnDevice(newVal);
        }
        this.previousBladeAngle = newVal;
    },
    setAngleOnDevice: function(angle) {
        dSS.util.genericRequest({
                url: '/json/device/setConfig',
                scope: this,
                params:
                {
                    'class': 64,
                    index: 0x04,
                    dsuid: this.dSUID,
                    value: Math.round(angle / 100 * 0xff)
                },
                failureMsg: dSS._("Could not set device blade angle")
            });
    },
    handleMax: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.MAX);
        } else {
            this.fireEvent('formchanged');
        }
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
        this.dirtyValue = true;
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(100);
            this.angleSlider.setValue(100);
            this.dirtyAngle = false;
            if (this.apartmentMode !== true) {
                this.handleBladeAngleChanged.defer(300, this,
                        [ this.bladeAngleField, 100 ]);
            }
        }
    },
    handleMin: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.MIN);
        } else {
            this.fireEvent('formchanged');
        }
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.dirtyValue = true;
        if (this.matrix.shadebladeang === true) {
            this.bladeAngleField.setValue(0);
            this.angleSlider.setValue(0);
            this.dirtyAngle = false;
            if (this.apartmentMode !== true) {
                this.handleBladeAngleChanged.defer(300, this,
                        [ this.bladeAngleField, 0 ]);
            }
        }
    },
    handleStop: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
            dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP);
        } else {
            this.fireEvent('formchanged');
        }
        if ((this.apartmentMode !== true) &&
            (this.matrix.shadeposition === true)) {
            this.dirtyValue = true;
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
        }
    },
    handleInc: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
        } else {
            this.fireEvent('formchanged');
        }
            
        this.dirtyValue = true;

        if ((this.matrix.shadeposition === true) && 
            (this.apartmentMode !== true)) {

            dSS.util.callScene(this.dSUID, dSS.config.scenes.INC);
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
            return;
        }

        var v = this.valueSlider.getValue() + 5;
        if (v > 100) {
            v = 100;
        }

        if (v != this.valueSlider.getValue()) {
            this.valueSlider.setValue(v);
            this.handleValueChanged(this.valueSlider, v);
        }
    },
    handleDec: function() {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
            this.killRequests();
        } else {
            this.fireEvent('formchanged');
        }

        this.dirtyValue = true;

        if ((this.matrix.shadeposition === true) &&
            (this.apartmentMode !== true)) {

            dSS.util.callScene(this.dSUID, dSS.config.scenes.DEC);
            this.readDeviceValue.defer(500, this, [ this.dSUID ]);
            return;
        }

        var v = this.valueSlider.getValue() - 5;
        if (v < 0) {
            v = 0;
        }

        this.handleValueChanged(this.valueSlider, v);
    },
    handleForcedModeOn: function() {
        if (this.apartmentMode === true) {
            return;
        }
        dSS.util.callScene(this.dSUID, dSS.config.scenes.LOCAL_ON); // 51
        this.readOperationMode.defer(500, this, [ this.dSUID ]);
    },
    handleForcedModeOff: function() {
        if (this.apartmentMode === true) {
            return;
        }
        dSS.util.callScene(this.dSUID, dSS.config.scenes.LOCAL_OFF); // 50
        this.readOperationMode.defer(500, this, [ this.dSUID ]);
    },
    handleNoForcedMode: function() {
        if (this.apartmentMode === true) {
            return;
        }
        dSS.util.callScene(this.dSUID, dSS.config.scenes.STOP); // 15
        this.readOperationMode.defer(500, this, [ this.dSUID ]);
    },
    clearData: function()
    {
        this.killRequests();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);
        this.dirtyValue = false;
        this.dirtyAngle = false;
        this.previousValue = 0;
        this.previousBladeAngle = 0;
    },
    killRequests: function() {
		this.fireEvent('loading', false);
        if (this.AjaxRequestValue !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestValue);
            this.AjaxRequestValue = undefined;
        }

        if (this.AjaxRequestAngle !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestAngle);
            this.AjaxRequestAngle = undefined;
        }

        if (this.AjaxRequestOperationMode !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestOperationMode);
            this.AjaxRequestOperationMode = undefined;
        }
    },
    readBladeAngleValue: function(reqdSUID) {
        this.killRequests();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        var request = 'getConfig';
        var offset = 0x04;

        this.dirtyAngle = false;
        this.AjaxRequestAngle = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);
                        this.bladeAngleField.markInvalid(
                            dSS._("Could not load current device blade angle position") +
                            ': ' + jsonData.message);

                    } else if (!isNaN(jsonData.result.value)) {
                        this.previousBladeAngle = Math.round(jsonData.result.value * 100 / 0xff);
                        this.bladeAngleField.setValue(this.previousBladeAngle);
                        this.angleSlider.setValue(this.previousBladeAngle);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestAngle = undefined;
                if ((this.closed === true) ||
                    (this.dirtyAngle === true)  ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        jsonData.message);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + jsonData.message);

                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device blade angle position") + ': ' +
                        err);
                    this.bladeAngleField.markInvalid(
                        dSS._("Could not load current device blade angle position") +
                        ': ' + err);
                }
            }
        });
    },
    readOperationMode: function(reqdSUID) {
        this.killRequests();

        if (reqdSUID !== this.dSUID) {
            return;
        }

        this.fireEvent('loading', true);
        this.opModeText.setText(dSS._("loading") + '...');
        var request = 'getConfig';
        var offset = 0x05;

        this.AjaxRequestOperationMode = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: reqdSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device operation mode") +
                            ': ' + jsonData.message);
                        this.opModeText.setText("");

                    } else if (!isNaN(jsonData.result.value)) {
                        var text = dSS.config.HeatingOperationModeStrings[jsonData.result.value];
                        this.opModeText.setText(dSS._(text));
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        err);
                    this.opModeText.setText("");
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestOperationMode = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        jsonData.message);
                    this.opModeText.setText("");
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device operation mode") + ': ' +
                        err);
                    this.opModeText.setText("");
                }
            }
        });
    },
    loadData: function(reqdSUID, devClass, matrix, virtualDevice)
    {
        this.killRequests();

        this.dSUID = reqdSUID;

        this.matrix = matrix;

        this.fireEvent('loading', true);

        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
        this.bladeAngleField.setValue(0);
        this.angleSlider.setValue(0);

        if ((matrix.shadeposition == true) && (matrix.outvalue8 === true)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Unsupported configuration"));
            return;
        }

        this.fireEvent('enableme');

        this.visualConfig(matrix, devClass, virtualDevice);

        this.readDeviceValue(reqdSUID);
    },
    readDeviceValue: function(dSUID) {
        var request = 'getConfigWord';
        var offset = 0x02;

        if (this.maxValue <= 0xff) {
            request = 'getConfig';
            offset = 0;
        }

        if (this.matrix.pwmvalue === true) {
            request = 'getConfig';
            offset = 4;
        }

        this.dirtyValue = false;
        this.dirtyAngle = false;

        this.AjaxRequestValue = Ext.Ajax.request({
            url: '/json/device/' + request,
            method: 'GET',
            params:
            {
                'class': 64,
                index: offset,
                dsuid: dSUID
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if ((this.closed === true) ||
                    (this.dirtyValue === true)  ||
                    (dSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                        this.valueField.markInvalid(
                            dSS._("Could not load current device value") +
                            ': ' + jsonData.message);
                    } else if (!isNaN(jsonData.result.value)) {
                        var p;
                        if (this.matrix.pwmvalue === true) {
                            p = jsonData.result.value;
                        } else {
                            p = dSS.util.valueToPercent(jsonData.result.value,
                                                        this.maxValue);
                        }
                        this.valueSlider.setValue(p);
                        this.valueField.setValue(p);
                        this.previousValue = p;
                    }

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(dSUID);
                    }
                    if (this.matrix.pwmvalue === true) {
                        this.readOperationMode(dSUID);
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestValue = undefined;
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);

                    if (this.matrix.shadebladeang === true) {
                        this.readBladeAngleValue(dSUID);
                    }
                    if (this.matrix.pwmvalue === true) {
                        this.readOperationMode(dSUID);
                    }
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current device value") + ': ' +
                        err);
                    this.valueField.markInvalid(
                        dSS._("Could not load current device value") + ': ' +
                        err);
                }
            }
        });
    },
    setDeviceValue: function(value)
    {
        if (isNaN(value)) {
            return;
        }

        var p = dSS.util.valueToPercent(value, this.maxValue);
        this.valueSlider.setValue(p);
        this.valueField.setValue(p);
    },
    setBladeAngle: function(angle) {
        if (isNaN(angle)) {
            return;
        }

        this.previousBladeAngle = angle;

        var p = Math.round(dSS.util.valueToPercent(angle, 0xff));
        this.bladeAngleField.setValue(p);
        this.angleSlider.setValue(p);
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }

        if (this.apartmentMode !== true) {
            return;
        }

        var value = dSS.util.percentToValue(this.valueField.getValue(), this.maxValue);
        var angle = dSS.util.percentToValue(this.bladeAngleField.getValue(), 0xff);

        if (this.matrix.shadebladeang !== true) {
            angle = undefined;
        }
        dSS.util.setSceneValue.defer(500, this,
                [this.dSUID, scene, value, angle ]);
        this.resetDefaults();
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }
        this.previousValue = dSS.util.percentToValue(this.valueField.getValue(), this.maxValue);
        if (this.matrix.shadebladeang) {
            this.previousBladeAngle = dSS.util.percentToValue(this.bladeAngleField.getValue(), 0xff);
        }
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }
        var p = dSS.util.valueToPercent(this.previousValue, this.maxValue);
        this.valueField.setValue(p);
        this.valueSlider.setValue(p);

        if (this.matrix.shadebladeang) {
            var p = dSS.util.valueToPercent(this.previousBladeAngle, 0xff);
            this.bladeAngleField.setValue(p);
            this.bladeAngleSlider.setValue(p);
        }
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }

        var value = this.valueField.getValue();
        this.setValueOnDevice(value);

        if (this.matrix.shadebladeang === true) {
            var angle = this.bladeAngleField.getValue();
            this.setAngleOnDevice(angle);
        }
    },
    visualConfig: function(matrix, devClass, virtualDevice) {
        this.matrix = matrix;

        if (matrix.shadeposition === true) {
            this.maxValue = 0xffff;
        } else {
            this.maxValue = 0xff;
        }

        if (matrix.shadeposition === true) {
            this.valueLabel.setText(dSS._("Position") + ':');
        } else {
            this.valueLabel.setText(dSS._("Value") + ':');
        }

        this.bladeAngleLabel.setVisible(matrix.shadebladeang);
        this.bladeAngleUnit.setVisible(matrix.shadebladeang);
        this.bladeAngleField.setVisible(matrix.shadebladeang);
        this.angleSlider.setVisible(matrix.shadebladeang);
        this.grKLHack.setVisible((this.apartmentMode === true) &&
                                 (matrix.grkl387workaround === true));

        if ((devClass === dSS.config.deviceClass.GR) ||
            (devClass === dSS.config.deviceClass.GN) ||
            (devClass === dSS.config.deviceClass.RT)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }

		if (devClass === dSS.config.deviceClass.GR) {
            this.minButton.setText(dSS._("Down"));
            this.maxButton.setText(dSS._("Up"));
            this.forcedModeOnButton.hide();
            this.forcedModeOffButton.hide();
            this.noForcedModeButton.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            if ((this.apartmentMode === true) &&
                (matrix.grkl387workaround === true)) {
                this.valueSlider.increment = dSS.config.GRKL387SliderIncrement;
                this.valueSlider.keyIncrement = dSS.config.GRKL387SliderIncrement;
            } else {
                this.valueSlider.increment = this.decPrecision;
                this.valueSlider.keyIncrement = this.decPrecision;
            }
            this.valueSlider.enable();
        } else if (devClass === dSS.config.deviceClass.BL) {
            this.maxButton.hide();
            this.minButton.hide();
            this.decButton.hide();
            this.incButton.hide();
            this.forcedModeOnButton.show();
            this.forcedModeOffButton.show();
            this.noForcedModeButton.show();
            if (virtualDevice === true) {
                this.forcedModeOnButton.disable();
                this.forcedModeOffButton.disable();
                this.noForcedModeButton.disable();
                this.opModeLabel.hide();
                this.opModeText.hide();
            } else {
                this.opModeLabel.show();
                this.opModeText.show();
                this.opModeText.setText(dSS._("loading") + '...');
            }
            this.valueField.disable();
            this.valueSlider.disable();
        } else {
            this.minButton.setText(dSS._("Min"));
            this.maxButton.setText(dSS._("Max"));
            this.forcedModeOnButton.hide();
            this.forcedModeOffButton.hide();
            this.noForcedModeButton.hide();
            this.opModeLabel.hide();
            this.opModeText.hide();
            this.maxButton.show();
            this.minButton.show();
            this.decButton.show();
            this.decButton.show();
            this.incButton.show();
            this.valueField.enable();
            this.valueSlider.enable();
        }
    }
});

dSS.devices.DeviceSettingsFCUView = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        var xPos = config.xPos;
        var xPosOffset = config.xPosOffset;

        this.closed = false;

        this.dSUID = config.dSUID;

        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.AjaxRequestFCUOutputChannels = undefined;

        this.dirtyFanSpeed = false;
        this.dirtyLouverPos = false;

        this.previousairflowintensity = 0;
        this.previousLouverPosition = 0;
        this.previousautomatic = false;
        this.previousswing = false;
        this.previousfanretain = false;
        this.previouslouverretain = false;

        this.fanspeedslider = new Ext.slider.SingleSlider({
            height: 70,
            value: 0,
            minValue: 0,
            maxValue: 100,
            vertical: true,
            disabled: true,
            increment: 1,
            allowDecimals: false,
            decimalPrecision: 0,
            forceDecimalPrecision: true,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleFanSpeedChanged.createDelegate(this)
            }
        });

        this.louverpositionslider = new Ext.slider.SingleSlider({
            height: 70,
            value: 0,
            minValue: 0,
            maxValue: 100,
            vertical: true,
            disabled: true,
            increment: 1,
            allowDecimals: false,
            decimalPrecision: 0,
            forceDecimalPrecision: true,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleLouverPositionChanged.createDelegate(this)
            }
        });

        var directionitems = [];

        var group;

        if (config.device !== undefined) {
            group = dSS.util.getJokerGroup(config.device.get('groups'));
        }

        if (group == dSS.config.groups.RECIRCULATION) {
            directionitems =
            [
                [
                    0,
                    dSS._("no direction defined")
                ],
                [
                    1,
                    dSS._("down")
                ],
                [
                    2,
                    dSS._("up")
                ]
            ];
        } else { // ventilation
            directionitems =
            [
                [
                    0,
                    dSS._("supply and exhaust air")
                ],
                [
                    1,
                    dSS._("supply air")
                ],
                [
                    2,
                    dSS._("exhaust air")
                ]
            ];
        }

        this.airflowdirection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            disabled: true,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: directionitems
            }),
            valueField: 'id',
            displayField: 'displayText',
            valueNotFoundText: " ",
            value: 0
        });

        Ext.apply(this, {
            layout: 'absolute',
            border: false,
            padding: 5,
            bubbleEvents: [ 'formchanged', 'loading', 'valuechanged',
                            'enableme'  ],
            items:
            [
                // ventilation part
                {
                    xtype: 'label',
                    ref: 'fanspeedlabel',
                    text: dSS._("Ventilation speed") + ':',
                    disabled: true,
                    x: 10,
                    y: 16
                },
                {
                    xtype: 'numberfield',
                    ref: 'fanspeedvalue',
                    disabled: true,
                    x: xPos - xPosOffset,
                    y: 12,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleFanSpeedChanged.createDelegate(this)
                    }
                },
                {
                    xtype:'label',
                    ref: 'fanspeedpercent',
                    disabled: true,
                    x: xPos - (xPosOffset - 56),
                    y: 16,
                    text: '%'
                },
                // this is a workaround for a bug when the slider thumb was
                // rendered at a wrong position; I could not figure out why
                // this is happening, but it is related to who is rendered
                // when/hidden rendering, so the workaround is to render only a
                // container here and attach the slider upon view activation
                {
                    xtype: 'container',
                    ref: 'fanspeedslidercontainer',
                    disabled: true,
                    x: xPos - (xPosOffset - 70),
                    y: 12,
                    height: 70
                },
                {
                    xtype: 'button',
                    x: xPos - (xPosOffset - 102),
                    y: 12,
                    width: 130,
                    ref: 'ventmax',
                    disabled: true,
                    text: dSS._("Max"),
                    handler: this.handleFanSpeedMaxButton.createDelegate(this)
                },
                {
                    xtype: 'button',
                    x: xPos - (xPosOffset - 102),
                    y: 60,
                    width: 130,
                    ref: 'ventmin',
                    disabled: true,
                    text: dSS._("Min"),
                    handler: this.handleFanSpeedMinButton.createDelegate(this)
                },
                {
                    xtype: 'checkbox',
                    ref: 'fcuautomatic',
                    disabled: true,
                    x: 50,
                    y: 35,
                    handler: this.handleAutomaticSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'automaticlabel',
                    text: dSS._("Automatic"),
                    disabled: true,
                    x: 70,
                    y: 40
                },
                {
                    xtype: 'checkbox',
                    ref: 'fcufanretainoutput',
                    disabled: true,
                    x: 50,
                    y: 65,
                    hidden: (this.apartmentMode != true),
                    handler: this.handleFanRetainOutputSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'fcufanretainoutputlabel',
                    text: dSS._("Retain output"),
                    x: 70,
                    y: 70,
                    disabled: true,
                    hidden: (this.apartmentMode != true)
                },
                // airflow direction part
                {
                    xtype: 'label',
                    ref: 'directionlabel',
                    disabled: true,
                    text: dSS._("Airflow direction") + ':',
                    x: 10,
                    y: 108
                },
                {
                    xtype: 'panel',
                    x: xPos - xPosOffset,
                    y: 105,
                    width: 232,
                    layout: 'fit',
                    border: false,
                    items: [ this.airflowdirection ]
                },
                {
                    xtype: 'checkbox',
                    ref: 'airflowdirectionretainoutput',
                    disabled: true,
                    x: 50,
                    y: 130,
                    hidden: (this.apartmentMode != true),
                    handler: this.handleAirflowDirectionRetainOutputSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    disabled: true,
                    ref: 'airflowdirectionretainoutputlabel',
                    text: dSS._("Retain output"),
                    x: 70,
                    y: 133,
                    hidden: (this.apartmentMode != true)
                },

                // louver part
                {
                    xtype: 'label',
                    ref: 'louverlabel',
                    disabled: true,
                    text: dSS._("Louver position") + ':',
                    x: 10,
                    y: 166
                },
                {
                    xtype: 'numberfield',
                    ref: 'louverpositionvalue',
                    disabled: true,
                    x: xPos - xPosOffset,
                    y: 162,
                    width: 50,
                    allowBlank: false,
                    allowNegative: false,
                    allowDecimals: false,
                    value: 0,
                    minValue: 0,
                    maxValue: 100,
                    listeners: {
                        change: this.handleLouverPositionChanged.createDelegate(this)
                    }
                },
                {
                    xtype:'label',
                    ref: 'louverpercent',
                    disabled: true,
                    x: xPos - (xPosOffset - 56),
                    y: 166,
                    text: '%'
                },
                {
                    xtype: 'container',
                    ref: 'louverpositionslidercontainer',
                    disabled: true,
                    x: xPos - (xPosOffset - 70), 
                    y: 162,
                    height: 70
                },
                {
                    xtype: 'button',
                    disabled: true,
                    x: xPos - (xPosOffset - 102),
                    y: 162,
                    width: 130,
                    ref: 'louvermax',
                    text: dSS._("Max"),
                    handler: this.handleLouverPositionMaxButton.createDelegate(this)
                },
                {
                    xtype: 'button',
                    disabled: true,
                    x: xPos - (xPosOffset - 102),
                    y: 206,
                    width: 130,
                    ref: 'louvermin',
                    text: dSS._("Min"),
                    handler: this.handleLouverPositionMinButton.createDelegate(this)
                },
                {
                    xtype: 'checkbox',
                    disabled: true,
                    ref: 'fcuswing',
                    x: 50,
                    y: 185,
                    handler: this.handleSwingSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'swinglabel',
                    disabled: true,
                    text: dSS._("Swing"),
                    x: 70,
                    y: 190
                },
                {
                    xtype: 'checkbox',
                    disabled: true,
                    ref: 'fculouverretainoutput',
                    x: 50,
                    y: 215,
                    hidden: (this.apartmentMode != true),
                    handler: this.handleLouverRetainOutputSelection.createDelegate(this)
                },
                {
                    xtype: 'label',
                    ref: 'fculouverretainoutputlabel',
                    text: dSS._("Retain output"),
                    disabled: true,
                    x: 70,
                    y: 220,
                    hidden: (this.apartmentMode != true)
                }
            ]
        });

        dSS.devices.DeviceSettingsFCUView.superclass.constructor.apply(this, arguments);

        this.fanspeedvalue.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var nv = parseFloat(field.getValue());
                var ov = parseFloat(this.fanspeedslider.getValue());
                if (nv !== ov) {
                    this.handleFanSpeedChanged(field, nv);
                }
            }
        },
        this);

        this.louverpositionvalue.on('specialkey', function(field, e) {
            if (e.getKey() == e.ENTER) {
                var nv = parseFloat(field.getValue());
                var ov = parseFloat(this.louverpositionslider.getValue());
                if (nv !== ov) {
                    this.handleLouverPositionChanged(field, nv);
                }
            }
        },
        this);

        this.on('beforedestroy', function() {
            this.killRequests();
            this.closed = true;
        },
        this);

        this.on('activate', function() {
            this.fanspeedslidercontainer.add(this.fanspeedslider);
            this.louverpositionslidercontainer.add(this.louverpositionslider);
        },
        this);

        this.airflowdirection.on('select',
                this.handleAirflowDirectionChanged.createDelegate(this), this); 
    },
    handleFanSpeedMaxButton: function() {
        this.fanspeedvalue.setValue(100);
        this.handleFanSpeedChanged(this.fanspeedvalue, 100);
    },
    handleFanSpeedMinButton: function() {
        this.fanspeedvalue.setValue(0);
        this.handleFanSpeedChanged(this.fanspeedvalue, 0);
    },
    handleLouverPositionMaxButton: function() {
        this.louverpositionvalue.setValue(100);
        this.handleLouverPositionChanged(this.louverpositionvalue, 100);
    },
    handleLouverPositionMinButton: function() {
        this.louverpositionvalue.setValue(0);
        this.handleLouverPositionChanged(this.louverpositionvalue, 0);
    },
    handleFanSpeedChanged: function(container, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
        this.killRequests();
        this.fanspeedvalue.setValue(newVal);
        this.fanspeedslider.setValue(newVal);

        if ((this.apartmentMode !== true) &&
            (this.previousFanSpeed != newVal)) {
            this.dirtyFanSpeed = true;
            this.setFanSpeedOnDevice(newVal);
            this.previousFanSpeed = newVal;
        }
    },
    setFanSpeedOnDevice: function(value) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
		channelvalues: 'airFlowIntensity=' + value,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set airflow intensity")
        });
    },
    handleAutomaticSelection: function(cb, checked) {
        if (checked) {
            this.disableFanSpeed();
        } else {
            this.enableFanSpeed();
        }
        if (this.apartmentMode !== true) {
            this.setAutomaticModeOnDevice('airFlowAuto', checked);
            this.dirtyAutomatic = true;
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleSwingSelection: function(cb, checked) {
        if (checked) {
            this.disableLouver();
        } else {
            this.enableLouver();
        }
        if (this.apartmentMode !== true) {
            this.setAutomaticModeOnDevice('airLouverAuto', checked);
            this.dirtySwing = true;
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleFanRetainOutputSelection: function(cb, checked) {
        if (this.apartmentMode !== true) {
            this.setRetainModeOnDevice('airFlowIntensity', checked);
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleLouverRetainOutputSelection: function(cb, checked) {
        if (this.apartmentMode !== true) {
            this.setRetainModeOnDevice('airLouverPosition', checked);
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleAirflowDirectionRetainOutputSelection: function(cb, checked) {
        if (this.apartmentMode !== true) {
            this.setRetainModeOnDevice('airFlowDirection', checked);
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    handleLouverPositionChanged: function(field, newVal) {
        if (this.apartmentMode !== true) {
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }

        this.killRequests();
        this.louverpositionvalue.setValue(newVal);
        this.louverpositionslider.setValue(newVal);

        if ((this.apartmentMode !== true) &&
            (this.previousLouverPosition !== newVal)) {
            this.dirtyLouverPosition = true;
            this.setLouverPositionOnDevice(newVal);
            this.previousLouverPosition = newVal;
        }
    },
    handleAirflowDirectionChanged: function(combo, record) {
        if (this.apartmentMode !== true) {
            this.setAirflowDirectionOnDevice(record.get('id'));
            this.dirtyDirection = true;
            this.fireEvent('valuechanged', this.dSUID);
        } else {
            this.fireEvent('formchanged');
        }
    },
    setLouverPositionOnDevice: function(value) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
		channelvalues: 'airLouverPosition=' + value,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set louver position")
        });
    },
    setAutomaticModeOnDevice: function(channel, checked) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: channel + '=' + (checked ? "1" : "0"),
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set selected mode on device")
        });
    },
    setRetainModeOnDevice: function(channel, checked) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelScene',
            scope: this,
            params:
            {
                channels: JSON.stringify(parameter),
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set dontcare")
        });
    },
    setAirflowDirectionOnDevice: function(direction) {
        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue',
            scope: this,
            params:
            {
                channelvalues: 'airFlowDirection' + '=' + direction,
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set selected mode on device")
        });
    },
    enableFanSpeed: function() {
        this.fanspeedvalue.enable();
        this.fanspeedslider.enable();
        this.fanspeedlabel.enable();
        this.fanspeedpercent.enable();
        this.ventmax.enable();
        this.ventmin.enable();
    },
    disableFanSpeed: function() {
        this.fanspeedvalue.disable();
        this.fanspeedslider.disable();
        this.fanspeedlabel.disable();
        this.fanspeedpercent.disable();
        this.ventmax.disable();
        this.ventmin.disable();
    },
    enableLouver: function() {
        this.louverpositionvalue.enable();
        this.louverpositionslider.enable();
        this.louverlabel.enable();
        this.louverpercent.enable();
        this.louvermax.enable();
        this.louvermin.enable();
    },
    disableLouver: function() {
        this.louverpositionvalue.disable();
        this.louverpositionslider.disable();
        this.louverlabel.disable();
        this.louverpercent.disable();
        this.louvermax.disable();
        this.louvermin.disable();
    },
    enableDirection: function() {
        this.airflowdirection.enable();
        this.directionlabel.enable();
    },
    enableAuto: function() {
        this.fcuautomatic.enable();
        this.automaticlabel.enable();
    },
    enableSwing: function() {
        this.fcuswing.enable();
        this.swinglabel.enable();
    },
    enableFanSpeedRetain: function() {
        this.fcufanretainoutput.enable();
        this.fcufanretainoutputlabel.enable();
    },
    enableDirectionRetain: function() {
        this.airflowdirectionretainoutput.enable();
        this.airflowdirectionretainoutputlabel.enable();
    },
    enableLouverRetain: function() {
        this.fculouverretainoutput.enable();
        this.fculouverretainoutputlabel.enable();
    },
    loadData: function(reqdSUID) {
        this.killRequests();

        this.fireEvent('loading', true);

        this.fireEvent('enableme');

        this.dSUID = reqdSUID;

        this.dirtyFanSpeed = false;
        this.dirtyLouverPosition = false;
        this.dirtyAutomatic = false;
        this.dirtySwing = false;
        this.dirtyDirection = false;

        this.AjaxRequestFCUOutputChannels = Ext.Ajax.request({
            url: '/json/device/getOutputChannelValue2',
            method: 'GET',
            params:
            {
                dsuid: reqdSUID,
                channels: 'airFlowIntensity;airFlowAuto;airLouverPosition;airLouverAuto;airFlowDirection'
            },
            scope: this,
            success: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestFCUOutputChannels = undefined;
                if ((this.closed === true) ||
                    (reqdSUID !== this.dSUID)) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    if (jsonData.ok !== true) {
                        dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Could not load current output channel " +
                                  "values") + ': ' + jsonData.message);
                        if (this.dirtyFanSpeed != true) {
                            this.fanspeedvalue.markInvalid();
                        }
                        if (this.dirtyairLouverPosition != true) {
                            this.louverpositionvalue.markInvalid();
                        }
                        return;
                    }

                    var airflow;
                    var louver;
                    var direction;
                    var swing = false;
                    var automatic = false;

                    if (jsonData.result.channels.airFlowIntensity !== undefined) {
                        var chan = jsonData.result.channels.airFlowIntensity;
                        airflow = Math.round(chan.value);
                        if (jsonData.result.channels.airFlowAuto !== undefined) {
                            var chan = jsonData.result.channels.airFlowAuto;
                            if (chan.value !== undefined) {
                                automatic = chan.value > 0;
                            }
                            this.enableAuto();
                        }
                        if (automatic != true) {
                            this.enableFanSpeed();
                        }
                    }

                    if (jsonData.result.channels.airLouverPosition !== undefined) {
                        var chan = jsonData.result.channels.airLouverPosition;
                        louver = Math.round(chan.value);
                        if (jsonData.result.channels.airLouverAuto !== undefined) {
                            var chan = jsonData.result.channels.airLouverAuto;
                            if (chan.value !== undefined) {
                                swing = chan.value > 0;
                            }
                            this.enableSwing();
                        }
                        if (swing != true) {
                            this.enableLouver();
                        }
                    }
                    if (jsonData.result.channels.airFlowDirection !== undefined) {
                        var chan = jsonData.result.channels.airFlowDirection;
                        if (chan.value !== undefined) {
                            direction = chan.value;
                        }
                        this.enableDirection();
                    }

                    if (this.dirtyFanSpeed != true) {
                        if (!isNaN(airflow)) {
                            this.previousairflowintensity = airflow;
                            this.fanspeedvalue.setValue(airflow);
                            this.fanspeedslider.setValue(airflow);
                        } else {
                            this.fanspeedvalue.markInvalid();
                        }
                    }

                    if (this.dirtyAutomatic != true) {
                        this.previousautomatic = automatic;
                        this.fcuautomatic.setValue(automatic);
                    }

                    if (this.dirtyLouverPosition != true) {
                        if (!isNaN(louver)) {
                            this.previousLouverPosition = louver;
                            this.louverpositionvalue.setValue(louver);
                            this.louverpositionslider.setValue(louver);
                        } else {
                            this.louverpositionvalue.markInvalid();
                        }
                    }

                    if (this.dirtySwing != true) {
                        this.prevousswing = swing;
                        this.fcuswing.setValue(swing);
                    }

                    if (this.dirtyDirection != true) {
                        if (!isNaN(direction)) {
                            this.previousdirection = direction;
                            this.airflowdirection.setValue(direction);
                        }
                    }
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            },
            failure: function(response) {
                this.fireEvent('loading', false);
                this.AjaxRequestFCUOutputChannels = undefined;
                if ((this.closed === true) || 
                    ((this.dirtyFanSpeed === true) &&
                     (this.dirtyLouverPosition === true))) {
                    return;
                }

                this.fireEvent('enableme');
                try {
                    var jsonData = Ext.util.JSON.decode(response.responseText);
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + jsonData.message);
                } catch (err) {
                    dSS.MessageBus.fireEvent('statusmessage',
                        dSS._("Could not load current output channel values") +
                        ': ' + err);
                }
            }
        });
    },
    killRequests: function() {
        if (this.AjaxRequestFCUOutputChannels !== undefined) {
            Ext.Ajax.abort(this.AjaxRequestFCUOutputChannels);
            this.AjaxRequestFCUOutputChannels = undefined;
        }
    },
    clearData: function() {
        this.killRequests();
        this.fanspeedvalue.setValue(0);
        this.fanspeedslider.setValue(0);
        this.louverpositionvalue.setValue(0);
        this.louverpositionslider.setValue(0);
        this.dirtyFanSpeed = false;
        this.dirtyLouverPos = false;
    },
    saveSceneValues: function(scene) {
        if (!this.isVisible()) {
            return;
        }

        if (this.apartmentMode !== true) {
            return;
        }

        this.killRequests();
        
        var parameter = {};

        if (this.airflowdirection.disabled != true) {
            parameter["airFlowDirection"] = {};
            parameter["airFlowDirection"]["value"] = this.airflowdirection.getValue();
            parameter["airFlowDirection"]["dontCare"] = this.airflowdirectionretainoutput.getValue();
        }

        if (this.fcuautomatic.disabled != true) {
            var fs = this.fanspeedvalue.getValue();
            parameter["airFlowAuto"] = {};
            parameter["airFlowAuto"]["value"] = this.fcuautomatic.getValue() ? 1 : 0;
            parameter["airFlowIntensity"] = {};
            if (this.fcuautomatic.getValue() != true) {
                parameter["airFlowIntensity"]["value"] = fs;
            }
            parameter["airFlowIntensity"]["dontCare"] = this.fcufanretainoutput.getValue();
        }

        if (this.fcuswing.disabled != true) {
            var lp = this.louverpositionvalue.getValue();
            parameter["airLouverAuto"] = {};
            parameter["airLouverAuto"]["value"] = this.fcuswing.getValue() ? 1 : 0;
            parameter["airLouverPosition"] = {};
            if (this.fcuswing.getValue() != true) {
                parameter["airLouverPosition"]["value"] = lp;
            }
            parameter["airLouverPosition"]["dontCare"] = this.fculouverretainoutput.getValue();
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelSceneValue2',
            scope: this,
            params:
            {
                channels: JSON.stringify(parameter),
                dsuid: this.dSUID,
                sceneNumber: scene
            },
            failureMsg: dSS._("Could not set louver position")
        });
        this.resetDefaults();
    },
    resetDefaults: function() {
        if (!this.isVisible()) {
            return;
        }

        this.previousairflowintensity = this.fanspeedvalue.getValue();
        this.previousLouverPosition = this.louverpositionvalue.getValue();
        this.previousautomatic = this.fcuautomatic.getValue();
        this.previousswing = this.fcuswing.getValue();
        this.previousdirection = this.airflowdirection.getValue();
    },
    resetValues: function() {
        if (!this.isVisible()) {
            return;
        }

        this.fanspeedvalue.setValue(this.previousairflowintensity);
        this.fanspeedslider.setValue(this.previousairflowintensity);

        this.louverpositionvalue.setValue(this.previousLouverPosition);
        this.louverpositionslider.setValue(this.previousLouverPosition);

        this.fcuautomatic.setValue(this.previousautomatic);
        this.fcuswing.setValue(this.previousswing);
        this.airflowdirection.setValue(this.previousdirection);
    },
    doPreview: function() {
        if (!this.isVisible()) {
            return;
        }

        var parameter = {};
        if (this.fanspeedvalue.disabled != true) {
            if ((this.fcuautomatic.disabled != true) &&
                (this.fcuautomatic.getValue()) == true) {
                parameter["airFlowAuto"] = true;
            } else {
                var fs = this.fanspeedvalue.getValue();
                parameter["airFlowIntensity"] = fs;
                if (this.fcuautomatic.disabled != true) {
                    parameter["airFlowAuto"] = false;
                }
            }
        }

        if (this.louverpositionvalue.disabled != true) {
            if ((this.fcuswing.disabled != true) &&
                (this.fcuswing.getValue() == true)) {
                parameter["airLouverAuto"] = true;
            } else {
                var lp = this.louverpositionvalue.getValue();
                parameter["airLouverPosition"] = lp;
                if (this.fcuswing.disabled != true) {
                    parameter["airLouverAuto"] = false;
                }
            }
        }

        if (this.airflowdirection.disabled != true) {
            parameter["airFlowDirection"] = this.airflowdirection.getValue();
        }

        dSS.util.genericRequest({
            url: '/json/device/setOutputChannelValue2',
            scope: this,
            params:
            {
                channels: JSON.stringify(parameter),
                applyNow: true,
                dsuid: this.dSUID
            },
            failureMsg: dSS._("Could not set selected mode on device")
        });
    },
    setLouverPosition: function(position) {
        this.enableLouver();
        if (isNaN(position)) {
            return;
        }

        this.louverpositionvalue.setValue(position);
        this.louverpositionslider.setValue(position);
    },
    setAirflowIntensity: function(speed) {
        this.enableFanSpeed();

        if (isNaN(speed)) {
            return;
        }
        this.fanspeedvalue.setValue(speed);
        this.fanspeedslider.setValue(speed);
    },
    setFanAutomatic: function(automatic) {
        this.enableAuto();
        this.fcuautomatic.setValue(automatic);
    },
    setLouverSwing: function(swing) {
        this.enableSwing();
        this.fcuswing.setValue(swing);
    },
    setFanRetainOutput: function(retain) {
        this.enableFanSpeedRetain();
        this.fcufanretainoutput.setValue(retain);
    },
    setLouverRetainOutput: function(retain) {
        this.enableLouverRetain();
        this.fculouverretainoutput.setValue(retain);
    },
    setAirFlowDirection: function(direction) {
        this.enableDirection();
        this.airflowdirection.setValue(direction);
    },
    setAirFlowDirectionRetainOutput: function(retain) {
        this.enableDirectionRetain();
        this.airflowdirectionretainoutput.setValue(retain);
    }
});

// devClass
//  stop button will be shown depending on the device class
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.addEvents('loading');
        this.notFirstCall = false;
        this.dSUID = config.dSUID;
        this.scene = config.scene;
        this.dummy = false;
        if (config.dummy !== undefined) {
            this.dummy = config.dummy;
        }

        this.maxValue = 0xff;
        var decPrecision = 0;

        if (config.matrix.shadeposition === true) {
            this.maxValue = 0xffff;
            decPrecision = 2;
        }

        this.apartmentMode = false;
        if (config.apartmentMode !== undefined) {
            this.apartmentMode = config.apartmentMode;
        }

        this.matrix = config.matrix;
        this.virtualDevice = config.virtualDevice;

        this.DEFAULT_HEIGHT = 176;
        this.EXTRA_HEIGHT = 210;
        this.EXTRA_HEIGHT_GRKL387HACK = this.EXTRA_HEIGHT + 25;
        this.EXTRA_HEIGHT_OUTPUT_CHANNELS = 305;
        this.EXTRA_HEIGHT_FCU = 246;
        var initialHeight = this.DEFAULT_HEIGHT;
        if (this.matrix.fcu === true) {
            initialHeight = this.EXTRA_HEIGHT_FCU;
        } else if (this.matrix.shadebladeang === true) {
            initialHeight = this.EXTRA_HEIGHT;
        } else if (this.matrix.outputchannels === true) {
            initialHeight = this.EXTRA_HEIGHT_OUTPUT_CHANNELS;
        } else if ((this.apartmentMode === true) &&
                   (this.matrix.grkl387workaround === true)) {
            initialHeight = this.EXTRA_HEIGHT_GRKL387HACK;
        }

        if (config.overrideHeight !== undefined) {
            this.EXTRA_HEIGHT = config.overrideHeight;
            initialHeight = config.overrideHeight;
        }


        var xPos = dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth');
        var xPosOffset = 280;

        this.noconfig = new Ext.Panel({
            x: 0,
            y: 0,
            border: false
        });

        this.colorPicker = new dSS.devices.DeviceSettingsHSVView({
            x: 0,
            y: 0,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID
        });

        this.configLink = new dSS.devices.DeviceSettingsVdcConfigLinkView({
            x: 0,
            y: 0,
            initialHeight: initialHeight,
            dSUID: this.dSUID,
            device: config.device
        });

        this.fcu = new dSS.devices.DeviceSettingsFCUView({
            x: 0,
            y: 0,
            xPos: xPos,
            xPosOffset: xPosOffset,
            height: this.EXTRA_HEIGHT_FCU,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID,
            device: config.device
        });

        this.sliderView = new dSS.devices.DeviceSettingsDefaultView({
            x: 0,
            y: 0,
            xPos: xPos,
            xPosOffset: xPosOffset,
            height: initialHeight,
            matrix: this.matrix,
            maxValue: this.maxValue,
            decPrecision: decPrecision,
            devClass: config.devClass,
            apartmentMode: this.apartmentMode,
            dSUID: this.dSUID,
        });

        Ext.apply(this, {
            layout: 'card',
            activeItem: 0,
            padding: 0,
            bubbleEvents: [ 'updateshadow', 'valuechanged', 'enableme',
                            'disableme', 'formchanged' ],
            height: initialHeight,
            items:
            [
                // do not reorder!
                this.noconfig,
                this.sliderView,
                this.colorPicker,
                this.fcu,
                this.configLink
            ]
        });



        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);

        this.addEvents('updateshadow');

        this.on('afterrender', function() {
                this.fireEvent('updateshadow', initialHeight);
                this.setData(this.dSUID, config.devClass, config.present,
                     config.matrix, config.virtualDevice, config.device);

            },
            this);

        this.on('beforedestroy', function() {
                this.killRequest();
                this.closed = true;
            },
            this);

        this.on('enableme', function() {
            if (this.apartmentMode !== true) {
                this.enable();
            }
        },
        this);
        this.on('disableme', function() {
            if (this.apartmentMode !== true) {
                this.disable();
            }
        },
        this);

    },
    visualConfig: function(matrix, virtualDevice) {
        if (this.apartmentMode !== true) {
            this.fireEvent('disableme');
        }
        if ((matrix.outvalue8 || matrix.shadeposition) &&
            (!matrix.outputchannels) && (!matrix.fcu)) {
            this.getLayout().setActiveItem(1); // default slider view
        } else if (matrix.outputchannels) {
            this.getLayout().setActiveItem(2); // color picker
	    } else if (matrix.fcu) {
            this.getLayout().setActiveItem(3); // fcu view
        } else if (virtualDevice === true) {
            this.getLayout().setActiveItem(4); // vdc configuration link
        } else {
            this.getLayout().setActiveItem(0);
        }

        this.doLayout(false, true);
    },
    setData: function(dSUID, devClass, present, matrix, virtualDevice, record) {
        this.dSUID = dSUID;
        this.matrix = matrix;

        if ((this.apartmentMode === true) &&
            (matrix.grkl387workaround === true)) {
            if (this.getHeight() < this.EXTRA_HEIGHT_GRKL387HACK) {
                this.setHeight(this.EXTRA_HEIGHT_GRKL387HACK);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_GRKL387HACK);
            }
        } else if (matrix.shadebladeang === true) {
            if (this.getHeight() < this.EXTRA_HEIGHT) {
                this.setHeight(this.EXTRA_HEIGHT);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT);
            }
        } else if (matrix.outputchannels === true) {
            if (this.getHeight() < this.EXTRA_HEIGHT_OUTPUT_CHANNELS) {
                this.setHeight(this.EXTRA_HEIGHT_OUTPUT_CHANNELS);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_OUTPUT_CHANNELS);
            }
        } else if (matrix.fcu === true) {
            if (this.getHeight() < this.EXTRA_HEIGHT_FCU) {
                this.setHeight(this.EXTRA_HEIGHT_FCU);
                this.fireEvent('updateshadow', this.EXTRA_HEIGHT_FCU);
            }
        }

        this.clearData();
        this.sliderView.visualConfig(matrix, devClass, virtualDevice);
        this.visualConfig(matrix, virtualDevice);

        if (!present) {
            return;
        }

        if ((matrix.outvalue8 || matrix.shadeposition) &&
            (!matrix.outputchannels) && (!matrix.fcu)) {
            this.sliderView.loadData(dSUID, devClass, matrix, virtualDevice);
        } else if (matrix.outputchannels) {
            this.colorPicker.loadData(dSUID);
	    } else if (matrix.fcu) {
            this.fcu.loadData(dSUID);
        } else if (virtualDevice === true) {
            this.configLink.loadData(dSUID, record, (this.notFirstCall != true));
        }

        this.doLayout(false, true);
        this.notFirstCall = true;
    },
    killRequest: function() {
        this.sliderView.killRequests();
        this.colorPicker.killRequests();
        this.configLink.killRequests();
        this.fcu.killRequests();
    },
    clearData: function() {
        this.sliderView.clearData();
        this.colorPicker.clearData();
        this.fcu.clearData();
    },
    doPreview: function() {
        this.colorPicker.doPreview();
        this.sliderView.doPreview();
        this.fcu.doPreview();
    },
    saveSceneValues: function() {
        if (this.apartmentMode !== true) {
            return;
        }
        this.colorPicker.saveSceneValues(this.scene);
        this.sliderView.saveSceneValues(this.scene);
        this.fcu.saveSceneValues(this.scene);
	},
    resetDefaults: function() {
        this.sliderView.resetDefaults();
        this.colorPicker.resetDefaults();
        this.fcu.resetDefaults();
    },
    resetValues: function() {
        this.sliderView.resetValues();
        this.colorPicker.resetValues();
        this.fcu.resetValues();
    }

});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);

dSS.devices.DeviceSettingsWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        this.HEIGHT_OFFSET = 130;

        if ((config.devices === undefined) || (config.index === undefined) ||
            (config.devices.length === 0)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No information for device"));
            return;
        }

        var mainPanel = new dSS.devices.DeviceSettingsRootPanel({
            devices: config.devices,
            index: config.index,
            rowSel: config.rowSel,
            border: false,
        });

        Ext.apply(this, {
            layout: 'fit',
            padding: 10,
            resizable: false,
            height: 306,
            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth'),
            title: dSS._("Device settings"),
            items: [ mainPanel ]
        });

        dSS.devices.DeviceSettingsWindow.superclass.constructor.apply(this,
                                                                     arguments);

        mainPanel.on('updatetitleicon', function(icon) {
            this.setIconClass(icon);
        },
        this);

        mainPanel.on('close', function() {
            this.close();
        },
        this);

        this.on('afterrender', function() {
            if (config.maskElement !== undefined) {
                config.maskElement.mask();
            }
        },
        this);

        this.on('close', function() {
            if (config.maskElement !== undefined) {
                config.maskElement.unmask();
            }
        },
        this);

        mainPanel.on('updateshadow', function(height) {
            this.setHeight(height + this.HEIGHT_OFFSET);
            var s = this.getEl().shadow;
            if (s) {
                s.realign(this.x, this.y, this.getWidth(), this.getHeight());
            }
        },
        this);

    }
});

// parameters:
//  maskElement
//  device record from the device store
dSS.devices.DeviceSettingsRootPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {

        if (config.rowSel !== undefined) {
            this.rowSel = config.rowSel;
        }

        var disableLeft = true;
        var disableRight = true;

        this.devices = config.devices;
        this.index = config.index;

        var device;
        if (this.devices.length > 0) {
            device = this.devices[this.index];
        }

        if (this.devices.length > 1) {
            if (this.index > 0) {
                disableLeft = false;
            }
            if (this.index < this.devices.length - 1) {
                disableRight = false;
            }
        }

        this.embedded = false;
        if (config.embedded === true) {
            this.embedded = true;
        }

        var devCls;
        if (device !== undefined) {
            var devInfo = dSS.util.parseProductID(device.get('productID'));
            devCls = dSS.util.parseFunctionID(device.get('functionID'));
            var matrix;
            try {
                matrix = dSS.devprops.GetVisibleProperties(device);
            } catch (error) {
                matrix = error;
            }
        } else {
            try {
                matrix = dSS.devprops.GetVisibleProperties();
            } catch (error) {
                matrix = error;
            }
        }

        this.left = new Ext.Button({
            iconCls: 'buttonLeftIcon',
            tooltip: dSS._("Select previous device"),
            width: 50,
            disabled: disableLeft,
            handler: this.previousDevice.createDelegate(this)
        });

        this.right = new Ext.Button({
            iconCls: 'buttonRightIcon',
            tooltip: dSS._("Select next device"),
            width: 50,
            disabled: disableRight,
            handler: this.nextDevice.createDelegate(this)
        });

        this.titleIcon = '';
        var enabled = false;
        var devName = " ";
        var devdSUID = "";
        var devVirtualDevice = false;
        var devDisplayID = "";
        var devPresent = false;
        var devClass = 0;

        if (device !== undefined) {
            this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
            enabled = ((device.get('isPresent') === true) &&
                       (matrix.outvalue8 || matrix.shadeposition));
            devName = device.get('name');
            devdSUID = device.get('dsuid');
            devDisplayID = device.get('DisplayID');
            devPresent = device.get('isPresent');
            devVirtualDevice = (device.get('VdcModelUID') != '');
            devClass = devCls[0];

            // output mode dependency, hack for BL-KM
            if (devClass === dSS.config.deviceClass.BL) {
                var devType = dSS.config.deviceTypes[devInfo[0]];
                if ((devType === "KM") && (devInfo[1] === 200)) {
                    var hack = this.getHeatingMods(matrix,
                                                   device.get('outputMode'));
                    matrix = hack.matrix;
                    devClass = hack.deviceClass;
                }
            }
        }

        var buttons =
        [
            this.left,
            this.right,
            new Ext.Toolbar.Fill()
        ];

        var bodyCss;
        var items = [];
        if (this.embedded === false) {
            buttons.push({
                xtype: 'button',
                text: dSS._("Close"),
                scope: this,
                handler: function() {
                    if (this.maskElement !== undefined) {
                        this.maskElement.unmask();
                    }
                    this.fireEvent('close');
                }
            });
            var info =
            {
                xtype: 'panel',
                ref: 'info',
                border: false,
                items:
                [
                    dSS.util.CreateDefaultDeviceInfoPanel(devName, devDisplayID)
                ]
            };

            items.push(info);
        } else {
            bodyCss = 'x-panel-no-left-border x-panel-no-right-border x-panel-no-top-border';
        }

        items.push(
        {
            xtype: 'dssdevsettingspanel',
            ref: 'settings',
            bodyCssClass: bodyCss,
            dSUID: devdSUID,
            DisplayID: devDisplayID,
            devClass: devClass,
            present: devPresent,
            matrix: matrix,
            virtualDevice: devVirtualDevice,
            device: device
        });

        Ext.apply(this, {
            bubbleEvents: [ 'updatetitleicon', 'close', 'updateshadow',
                            'valuechanged' ],
            layout: 'table',
            bodyBorder: false,
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            items: items,
            buttonAlign: 'left',
            buttons: buttons
        });

        dSS.devices.DeviceSettingsRootPanel.superclass.constructor.apply(this,
                                                                     arguments);
        this.on('afterrender', function() {
            var icon = enabled ? 'loading-indicator-small' : this.titleIcon;
            this.fireEvent('updatetitleicon', icon);
        },
        this);

        this.addEvents('updateshadow', 'updatetitleicon', 'close');

        if (this.maskElement !== undefined) {
            this.maskElement.mask();
        }

        this.on('beforedestroy', function(component) {
            if (this.maskElement !== undefined) {
                this.maskElement.unmask();
            }
        },
        this);

        this.settings.on('loading', function(l) {
            var icon;
            if (l === true) {
                icon = 'loading-indicator-small';
            } else {
                icon = this.titleIcon;
            }
            this.fireEvent('updatetitleicon', icon);
        },
        this);

    },
    setupButtons: function() {
        if (this.index == 0) {
            this.left.disable();
        }

        if ((this.devices.length > 1) &&
            (this.index < (this.devices.length - 1))) {
            this.right.enable();
        }

        if (this.index === (this.devices.length - 1)) {
            this.right.disable();
        }

        if ((this.devices.length > 1) && (this.index > 0)) {
            this.left.enable();
        }

    },
    loadDevices: function(devices) {
        this.devices = devices;
        this.right.disable();
        this.left.disable();
        if (this.devices.length > 0) {
            this.index = 0;
        }
    },
    setRowSelectionModel: function(model) {
        if (this.rowModel !== undefined) {
            this.rowModel.removeListener('rowselect',
                    this.loadRow.createDelegate(this), this);
        }
        this.rowSel = model;
        this.rowSel.addListener('rowselect',
                this.loadRow.createDelegate(this), this);
    },
    clearDialog: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            this.settings.clearData();
            if (!this.embedded) {
                this.info.removeAll(true);
                this.info.add(dSS.util.CreateDefaultDeviceInfoPanel(" ", " "));
                this.info.doLayout();
            }
        }
    },
    loadRow: function(model, index, record) {
        if (this.ownerCt.isVisible()) {
            if (record.get('isPresent') === false) {
                return;
            }
            if (this.index < this.devices.length) {
                this.index = index;
            }
            this.setupButtons();
            this.updateView(false);
        }
    },
    getHeatingMods: function(matrix, outputMode) {
        var cls = dSS.config.deviceClass.BL;
        var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(matrix));
        switch (outputMode) {
            case dSS.config.deviceParams.OUTPUT_MODE_SWITCH:
            case dSS.config.deviceParams.OUTPUT_MODE_PWM:
                cls = dSS.config.deviceClass.GE; // hack
                copy.heatinggroup = false;
                copy.heatingoutmode = false;
                copy.pwmvalue = false;
                break;
            default:
                break;
        }
        return { "deviceClass": cls, "matrix": copy };
    },
    updateView: function(doSelect) {
        var device = this.devices[this.index];
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        var matrix;
        try {
            matrix = dSS.devprops.GetVisibleProperties(device);
        } catch (error) {
            matrix = error;
        }

        this.titleIcon = 'titleIcon' + dSS.util.genCSSName(device);
        this.fireEvent('updatetitleicon', this.titleIcon);

        if (!this.embedded) {
            this.info.removeAll(true);
            this.info.add(
                dSS.util.CreateDefaultDeviceInfoPanel(device.get('name'),
                                                      device.get('DisplayID')));
            this.info.doLayout();
        }

        var devClass = devCls[0];
        // output mode dependency, hack for BL-KM
        if (devClass === dSS.config.deviceClass.BL) {
            var devType = dSS.config.deviceTypes[devInfo[0]];
            if ((devType === "KM") && (devInfo[1] === 200)) {
                var hack = this.getHeatingMods(matrix,
                                               device.get('outputMode'));
                matrix = hack.matrix;
                devClass = hack.deviceClass;
            }
        }
        this.settings.setData(device.get('dsuid'), devClass,
                              (device.get('isPresent') && (device.get('outputMode') != 0)),
                              matrix, device.get('isVdcDevice'), device);
        if ((this.rowSel !== undefined) && (doSelect !== false)) {
            this.rowSel.selectRow(this.index, false, false);
        }
    },
    previousDevice: function() {
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.left.disable();
            }

            if ((this.devices.length > 1) &&
                (this.index < (this.devices.length - 1))) {
                this.right.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.right.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.left.enable();
            }
        }
        this.updateView();
    }
});

Ext.reg('dssdevicesettingsrootpanel', dSS.devices.DeviceSettingsRootPanel);

dSS.devices.DeviceSettingsWrapper = Ext.extend(Ext.Panel, {
    constructor: function(config) {
        this.device = config.deviceRecord;
        var devCls = dSS.util.parseFunctionID(this.device.get('functionID'));
        this.matrix = config.visibilityMatrix;

        this.enabled = ((this.device.get('isPresent') === true) &&
                       (this.matrix.outvalue8 || this.matrix.shadeposition ||
                        this.matrix.outputchannels || this.matrix.fcu));

        this.scene = config.scene;

        this.resetButton = new Ext.Button({
            text: dSS._("Reset"),
            disabled: true,
            minWidth: 110,
            handler: this.resetSettings.createDelegate(this)
        });

        this.previewButton = new Ext.Button({
            text: dSS._("Preview"),
            minWidth: 110,
            disabled: !this.enabled,
            scope: this,
            handler: this.applySettings.createDelegate(this)
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formloaded', 'savebuttonstate', 'formsubmitted',
                            'formchanged' ],
            border: false,
            layout: 'border',
            height: 340,
            items:
            [
                {
                    xtype: 'dssdevsettingspanel',
                    ref: 'devsettings',
                    apartmentMode: true,
                    border: true,
                    bodyCssClass: 'x-panel-no-left-border x-panel-no-right-border x-panel-no-bottom-border',
                    dSUID: this.device.get('dsuid'),
                    DisplayID: this.device.get('DisplayID'),
                    devClass: devCls[0],
                    present: false,
                    matrix: this.matrix,
                    overrideHeight: 300,
                    region: 'north',
                    scene: config.scene,
                    device: this.device
                },
                {
                    xtype: 'panel',
                    region: 'center',
                    layout: 'vbox',
                    border: false,
                    ref: 'buttonpanel',
                    items:
                    [
                        {
                            xtype: 'box',
                            flex: 3
                        },
                        {
                            xtype: 'panel',
                            width: dSS.gthelper.getDialogConfig('deviceSettingsWindowWidth') - 40,
                            layout: 'hbox',
                            border: false,
                            items:
                            [
                                {
                                    xtype: 'box',
                                    width: 10
                                },
                                this.resetButton,
                                {
                                    xtype: 'box',
                                    flex: 3
                                },
                                this.previewButton,
                                {
                                    xtype: 'box',
                                    width: 10
                                }
                            ]
                        },
                        {
                            xtype: 'box',
                            height: 15
                        }
                    ]
                }
            ]
        });

        dSS.devices.DeviceSettingsWrapper.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('formchanged', function() {
            this.previewButton.enable();
            this.resetButton.enable();
        },
        this);

        this.on('enableme', function() {
            this.items.each(function(item) {
                item.enable();
            });
        },
        this, { buffer: 500 });

        this.on('dataloaded', function(data) {
            if ((data.loader !== dSS.devprops.Loaders.SCENE_VALUE) &&
                (data.loader !== dSS.devprops.Loaders.SCENE_VALUE_OUTPUT_CHANNELS) &&
                (data.loader !== dSS.devprops.Loaders.SCENE_VALUE2_OUTPUT_CHANNELS)) { 
                return;
            }

            if (data.json.ok !== true) {
                Ext.Msg.alert(dSS._("Error"), data.json.message);
                this.fireEvent('formloaded', false);
                return;
            }

            if (data.loader === dSS.devprops.Loaders.SCENE_VALUE) {
                this.devsettings.sliderView.setDeviceValue(data.json.result.value);

                if (data.json.result.angle !== undefined) {
                    this.devsettings.sliderView.setBladeAngle(data.json.result.angle);
                }
            } else if (data.loader === dSS.devprops.Loaders.SCENE_VALUE_OUTPUT_CHANNELS) {
                var hue;
                var sat;
                var val;

                var i;
                for (i = 0; i < data.json.result.channels.length; i++) {
                    var chan = data.json.result.channels[i];
                    if (chan.channel === "hue") {
                        hue = Math.round(chan.value * 10) / 10;
                    } else if (chan.channel === "saturation") {
                        sat = Math.round(chan.value * 10) / 10;
                    } else if (chan.channel === "brightness") {
                        val = Math.round(chan.value * 10) / 10;
                    }
                }

                if (this.matrix.outputchannels === true) {
                    if (!isNaN(hue) && !isNaN(sat) && !isNaN(val)) {
                        this.devsettings.colorPicker.setColor({ h: hue, s: sat, v: val });
                    }
                }
            } else if (data.loader === dSS.devprops.Loaders.SCENE_VALUE2_OUTPUT_CHANNELS) {
                var airflowintensity;
                var louverposition;
                var direction;
                var directiondc;
                var automatic;
                var fandc;
                var swing;
                var louverdc;

                var channels = data.json.result.scenes[this.scene].channels;
                if (channels.airFlowIntensity !== undefined) {
                    var chan = channels.airFlowIntensity;
                    if (chan.dontCare !== undefined) {
                        fandc = chan.dontCare;
                    }
                    if (chan.value !== undefined) {
                        airflowintensity = Math.round(chan.value * 10) / 10;
                    }
                }
                if (channels.airFlowAuto !== undefined) {
                    var chan = channels.airFlowAuto;
                    if (chan.value !== undefined) {
                        automatic = chan.value > 0;
                    }
                }
                if (channels.airLouverPosition !== undefined) {
                    var chan = channels.airLouverPosition;
                    if (chan.dontCare != undefined) {
                        louverdc = chan.dontCare;
                    }
                    if (chan.value != undefined) {
                        louverposition = Math.round(chan.value * 10) / 10;
                    }
                }
                if (channels.airLouverAuto !== undefined) {
                    var chan = channels.airLouverAuto;
                    if (chan.value !== undefined) {
                        swing = chan.value > 0;
                    }
                }

                if (channels.airFlowDirection !== undefined) {
                    var chan = channels.airFlowDirection;
                    if (chan.value !== undefined) {
                        direction = chan.value;
                    }
                    if (chan.dontCare != undefined) {
                        directiondc = chan.dontCare;
                    }
                }

                if (this.matrix.fcu) {
                    if (!isNaN(airflowintensity)) {
                        this.devsettings.fcu.setAirflowIntensity(airflowintensity);
                    }
                    if (!isNaN(louverposition)) {
                        this.devsettings.fcu.setLouverPosition(louverposition);
                    }
                    if (!isNaN(direction)) {
                        this.devsettings.fcu.setAirFlowDirection(direction);
                    }
                    if (automatic !== undefined) {
                        this.devsettings.fcu.setFanAutomatic(automatic);
                    }
                    if (fandc !== undefined) {
                        this.devsettings.fcu.setFanRetainOutput(fandc);
                    }
                    if (swing !== undefined) {
                        this.devsettings.fcu.setLouverSwing(swing);
                    }
                    if (louverdc !== undefined) {
                        this.devsettings.fcu.setLouverRetainOutput(louverdc);
                    }
                    if (directiondc !== undefined) {
                        this.devsettings.fcu.setAirFlowDirectionRetainOutput(directiondc);
                    }
                }
            }
            this.devsettings.resetDefaults();
            this.devsettings.visualConfig(this.matrix,
                                        (this.device.get('VdcModelUID') != ''));

            this.fireEvent('formloaded', true);
        },
        this);

        this.on('submit', function() {
            this.devsettings.saveSceneValues();
            this.resetButton.disable();
            this.fireEvent('formsubmitted');
        },
        this);

        dSS.MessageBus.on('deviceoutputmodechanged', function(checked) {
            // events arrive out of order, dataloaded might override
            // deviceoutputmodechanged and cause a wrong dialog status
            this.enabled = false;

            // can't say why but using 'this' object produced strange Ext
            // errors, tried to disable items separately, i.e. via
            // this.devsettings - but 'devsettings' object got lost from time
            // to time, which I also can't explain.
            // The workaround below seems to do the trick.
            if (checked) {
                this.items.each(function(item) {
                    item.disable();
                });
            } else {
                this.items.each(function(item) {
                    item.enable();
                });
            }
        },
        this);
    },
    applySettings: function() {
        this.devsettings.doPreview();
        this.previewButton.disable();
    },
    resetSettings: function() {
        this.devsettings.resetValues();
        this.previewButton.enable();
        this.resetButton.disable();
    }
});

Ext.reg('dssdevicesettingswrapper', dSS.devices.DeviceSettingsWrapper);

