Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.FTWBacklightTimeout = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.loadedValue = undefined;

        this.offpanel = new Ext.Panel(
        {
            border: false,
            bubbleEvents: [ 'change' ],
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Disable backlight in") + ":",
                    width: 150,
                    height: 35,
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'spinnerfield',
                    width: 60,
                    ref: 'backlight',
                    allowDecimals: false,
                    allowNegaive: false,
                    allowEmpty: false,
                    minValue: 1,
                    maxValue: 255
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'box',
                    width: 10
                }
            ]
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items: [ this.offpanel ]
        });

        dSS.devprops.FTWBacklightTimeout.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {

                this.offpanel.backlight.on('change', function(field, n, o) {
                        this.fireEvent('formchanged');
                    },
                    this);

               this.offpanel.backlight.on('spin', function() {
                         this.fireEvent('formchanged');
                    },
                    this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader != dSS.devprops.Loaders.FTW_DISPLAY_BACKLIGHT) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.offpanel.backlight.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                }

                this.loadedValue = data.json.result.seconds;
                this.offpanel.backlight.setValue(data.json.result.seconds);
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var seconds = this.offpanel.backlight.getValue();

            if ((seconds === undefined) || (seconds == this.loadedValue)) {
                this.fireEvent('formsubmitted');
                return; 
            }

            dSS.util.genericRequest.defer(2000, this,
            [
                {
                    url: '/json/device/setSK204BacklightTimeout',
                    params:
                    {
                        dsuid: this.dSUID,
                        seconds: seconds
                    },
                    failureMsg: dSS._("Could not configure backlight settings"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        },
        this);
    }
});

Ext.reg('dssdevftwbacklighttimeout', dSS.devprops.FTWBacklightTimeout);

