Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.FTWDisplaySettings = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');
        this.dialogType = config.type;
        this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.loadedValues = undefined;

        var checkboxes =
        [
            {
                boxLabel: dSS._("Show room set point temperature"),
                name: "roomsetpoint",
                id: "cb_room_setpoint"
            },
            {
                boxLabel: dSS._("Show humidity"),
                name: "humidity",
                id: "cb_humidity"
            }
        ];

        this.check = new Ext.form.CheckboxGroup(
        {
            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._("Display settings"),
            columns: 1,
            items: checkboxes
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items: this.check
        });

        dSS.devprops.FTWDisplaySettings.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.FTW_DISPLAY_SETTINGS) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.check.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.check.setValue(data.json.result);
                this.loadedValues = data.json.result;

                this.el.unmask();
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var setpoint = this.check.items.get('cb_room_setpoint').getValue();
            var humidity = this.check.items.get('cb_humidity').getValue();

            // build up parameters
            var dSUID = this.dSUID;

            var params = { dsuid : dSUID };

            var changed = false;

            if ((humidity !== undefined) &&
                (humidity !== this.loadedValues.humidity)) {
                params.humidity = humidity ? 1 : 0;
                changed = true;
            }

            if ((setpoint !== undefined) &&
                (setpoint !== this.loadedValues.roomsetpoint)) {
                params.roomsetpoint = setpoint ? 1 : 0;
                changed = true;
            }

            if (changed !== true) {
                this.fireEvent('formsubmitted');
                return;
            }

            Ext.Ajax.request({
                url: '/json/device/setSK204DisplayMode',
                method: 'GET',
                disableCaching: true,
                scope: this,
                params: params,
                scope: this,
                success: function(result, request) {
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (jsonData.ok !== true) {
                            throw "error";
                        }
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not save display mode settings"));
                    }
                    this.fireEvent('formsubmitted');
                },
                failure: function(result, request) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not save display mode settings"));
                    this.fireEvent('formsubmitted');
                }
            });
        },
        this);
    }
});

Ext.reg('dssdevftwdisplaysettings', dSS.devprops.FTWDisplaySettings);

