Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.FTWTempControlConfig = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');
        this.dialogType = config.type;
        this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.loadedValue = undefined;

        var items =
        [
            {
                boxLabel: dSS._("Temperature control"),
                name: 'rb_sk_control_' + this.dSUID,
                id: 'temperaturecontrol',
                inputValue : 'temperaturecontrol'
            },
            {
                boxLabel: dSS._("Temperature control and ventilation"),
                name: 'rb_sk_control_' + this.dSUID,
                id: 'temperaturecontrolventilation',
                inputValue : 'temperaturecontrolventilation'
            }
        ];

        this.radio = new Ext.form.RadioGroup(
        {
            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._("Mode"),
            columns: 1,
            items: items
        });


        Ext.apply(this, {
            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'formchanged', 'sharedata'
            ],
            padding: 5,
            scope: this,
            items: this.radio
        });

        dSS.devprops.FTWTempControlConfig.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.on('change', function(radio) {
                    this.fireEvent('formchanged');
                    this.fireEvent('sharedata',
                                   'ftwtempcontrolventilationselect',
                                   radio.getValue().getRawValue());
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.FTW_MODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.radio.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.radio.setValue(data.json.result.mode, true);
                this.loadedValue = data.json.result.mode;

                this.fireEvent('sharedata',
                               'ftwtempcontrolventilationselect',
                               this.radio.getValue().getRawValue());

                this.el.unmask();
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var value;
            // this happens if the value was not loaded correctly
            if (this.radio.getValue() != null) {
                value = this.radio.getValue().getRawValue();
            }

            // do not submit anything if value was not loaded and also has
            // not been changed by the user
            if ((value === undefined) || (value == this.loadedValue)) {
                this.fireEvent('formsubmitted');
                return;
            }

            var dSUID = this.dSUID;

            Ext.Ajax.request({
                url: '/json/device/setSK204Config',
                method: 'GET',
                disableCaching: true,
                scope: this,
                params:
                {
                    dsuid : dSUID,
                    mode : value
                },
                scope: this,
                success: function(result, request) {
                    try {
                        var jsonData = Ext.util.JSON.decode(result.responseText);
                        if (jsonData.ok !== true) {
                            throw "error";
                        }

                        dSS.util.processDeviceActionResponse(jsonData.result);
                        this.fireEvent('formsubmitted');
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not save device mode settings"));
                        this.fireEvent('formsubmitted');
                    }
                },
                failure: function(result, request) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not save device mode settings"));
                    this.fireEvent('formsubmitted');
                }
            });
        },
        this);
    }
});

Ext.reg('dssdevftwtempcontrolconfig', dSS.devprops.FTWTempControlConfig);

