Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received

dSS.devprops.InputTypeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.dSUID = config.dSUID;
        this.matrix = config.matrix;
        this.color = config.color;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var list =
        [
            [
                dSS.config.InputTypeHack.SENSOR,
                dSS._("Sensor")
            ]
        ];

        if (this.matrix.inputtypeonlysensor !== true) {
            list.push(
            [
                dSS.config.InputTypeHack.PUSHBUTTON,
                dSS._("Pushbutton")
            ]);
        }

        this.dropdown = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            fieldLabel: dSS._("Type"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'formsetdeviceconfig' ],
            padding: 5,
            scope: this,
            items: this.dropdown
        });

        dSS.devprops.InputTypeForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.dropdown.on('select', function(box, record, index) {
                    this.fireEvent('formchanged');
                },
                this);

                dSS.MessageBus.fireEvent('inputtypechanged', this.dropdown.getValue ());
            },
            this);

        this.dropdown.on('afterrender', function() {
            this.dropdown.on('select', function(box, record, index) {
                dSS.MessageBus.fireEvent('inputtypechanged', record.data.id);
            }, this);
        }, this);

        this.on('beforedestroy', function() {
                this.dropdown.removeListener('select', function(box, record, index) {
                    this.fireEvent('formchanged');
                },
                this);
                this.dropdown.removeListener('select', function(box, record, index) {
                    dSS.MessageBus.fireEvent('inputtypechanged', record.data.id);
                }, this);
        },
        this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }
            
                if (this.matrix.inputtypeonlysensor !== true) {
                    if (data.json.ok !== true) {
                        this.dropdown.markInvalid(data.json.message);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    // input type is either "Pushbutton" or "Sensor", can be 
                    // derived from the LT mode register.
                    // Pushbuttons are in the range of 0-14
                    // Sensors are in the range of >16
	
                    var val = data.json.result[this.dSUID][0].button[0].inputMode;
    				if (val <= dSS.config.deviceParams.BUTTONINPUT_1WAY) {
    	                this.dropdown.setValue(dSS.config.InputTypeHack.PUSHBUTTON);
                        this.loadedValue = dSS.config.InputTypeHack.PUSHBUTTON;
                    } else {
                        this.dropdown.setValue(dSS.config.InputTypeHack.SENSOR);
                        this.loadedValue = dSS.config.InputTypeHack.SENSOR;
                    }
                } else {
                    this.dropdown.setValue(dSS.config.InputTypeHack.SENSOR);
                    this.loadedValue = dSS.config.InputTypeHack.SENSOR;
                    this.disable();
                }

                this.el.unmask();
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            // this dropdown only provides the selection event for others,
            // there's nothing to submit, except when using the V2 API
            if (this.matrix.setumr200config === true) {
                var val = this.dropdown.getValue();
                if (val !== this.loadedValue) {
                    parameter = {};
                    if (this.matrix.jokerconfig != true) {
                        parameter.application = dSS.config.GroupV2Map[this.color];
                    }
                    parameter.inputType = dSS.config.InputTypeV2Map[val];
                    this.fireEvent('formsetdeviceconfig', parameter);
                    return;
                }
            }

            this.fireEvent('formsubmitted');
        },
        this);
    }
});

Ext.reg('dssdevinputtypeform', dSS.devprops.InputTypeForm);

