//= require <external/IconCombo>

Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID, group (configured joker group)
dSS.devprops.JokerConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dSUID === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSUID"));
            return;
        }

        this.dSUID = config.dSUID;
        this.device = config.device;
        this.group = config.group;
        this.matrix = config.matrix;

        this.userGroupWarned = false;

        var list =
        [
            [
                dSS.config.groups.LIGHT,
                dSS.config.GroupColorIconCls[dSS.config.groups.LIGHT],
                dSS._(dSS.config.groupNames[dSS.config.groups.LIGHT])
            ],
            [
                dSS.config.groups.SHADOW,
                dSS.config.GroupColorIconCls[dSS.config.groups.SHADOW],
                dSS._(dSS.config.groupNames[dSS.config.groups.SHADOW])
            ],
            [
                dSS.config.groups.HEATING,
                dSS.config.GroupColorIconCls[dSS.config.groups.HEATING],
                dSS._(dSS.config.groupNames[dSS.config.groups.HEATING])
            ],
            [
                dSS.config.groups.AUDIO,
                dSS.config.GroupColorIconCls[dSS.config.groups.AUDIO],
                dSS._(dSS.config.groupNames[dSS.config.groups.AUDIO])
            ],
            [
                dSS.config.groups.VIDEO,
                dSS.config.GroupColorIconCls[dSS.config.groups.VIDEO],
                dSS._(dSS.config.groupNames[dSS.config.groups.VIDEO])
            ],
            [
                dSS.config.groups.JOKER,
                dSS.config.GroupColorIconCls[dSS.config.groups.JOKER],
                dSS._(dSS.config.groupNames[dSS.config.groups.JOKER])
            ],
            [
                dSS.config.groups.VENTILATION,
                dSS.config.GroupColorIconCls[dSS.config.groups.VENTILATION],
                dSS._(dSS.config.groupNames[dSS.config.groups.VENTILATION])
            ],
            [
                dSS.config.groups.RECIRCULATION,
                dSS.config.GroupColorIconCls[dSS.config.groups.RECIRCULATION],
                dSS._(dSS.config.groupNames[dSS.config.groups.RECIRCULATION])
            ]
        ];

        if (this.matrix.apartmentapplication === true) {
            list.push(
            [
                dSS.config.groups.APT_VENTILATION,
                dSS.config.GroupColorIconCls[dSS.config.groups.APT_VENTILATION],
                dSS._(dSS.config.groupNames[dSS.config.groups.APT_VENTILATION])
            ]);
        }

        // #1891 - keep value but replace with "reserved" entry
        if ((config.group === dSS.config.groups.SECURITY) ||
            (config.group === dSS.config.groups.ACCESS)) {
            list.push(
                [
                    config.group,
                    '', // no icon
                    dSS._("Reserved")
                ]);
        }

        if (this.matrix.jokertempcontrol === true) {
            list.push(
                [
                    dSS.config.groups.TEMPCONTROL,
                    dSS.config.GroupColorIconCls[dSS.config.groups.TEMPCONTROL],
                    dSS._(dSS.config.groupNames[dSS.config.groups.TEMPCONTROL])
                ]);
        }

        this.addEvents('formsubmitted', 'formloaded');

        this.colorSelection = new Ext.ux.IconCombo({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'updateshadow' ],
            fieldLabel: dSS._("Color group"),
            tpl: new Ext.XTemplate(
                '<tpl for="."><div class="x-combo-list-item">',
                '<table><tbody><tr>',
                '<td>',
                '<div class="{css} x-icon-combo-icon"></div></td>',
                '<td>{displayText}</td>',
                '</tr></tbody></table>',
                '</div></tpl>'),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'css',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText',
            valueNotFoundText: " ",
            iconClsField: 'css'
        });

        Ext.apply(this, {
            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'formchanged', 'formdelayothers',
                'formsetdeviceconfig'
            ],
            padding: 5,
            height: 35,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.colorSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.JokerConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.colorSelection.on('select', function() {
                    this.fireEvent('formchanged');
                    if (this.matrix.setumr200config !== true) {
                        this.fireEvent('formdelayothers', this.getId());
                    }
                },
                this);
            },
            this);

        this.on('afterrender', function() {
                this.fireEvent('formloaded', true);
            },
            this);

        this.colorSelection.on('afterrender', function() {
            this.colorSelection.on('select', function(box, record, index) {
                    dSS.MessageBus.fireEvent('jokerconfigformchanged',
                                              record.data.id);
                },
                this);

                if ((this.closed !== true) &&
                   ((this.group >= dSS.config.groups.LIGHT) &&
                    (this.group <= dSS.config.groups.JOKER)) ||
                    (this.group === dSS.config.groups.APT_VENTILATION) ||
                    (this.group === dSS.config.groups.VENTILATION) ||
                    (this.group === dSS.config.groups.RECIRCULATION) ||
                    (this.group === dSS.config.groups.TEMPCONTROL)) {
                    this.colorSelection.setValue(this.group);
                }
            },
            this);

        this.colorSelection.on('expand', function(box, record, index) {
                var devGroups = this.device.get('groups');
                var name = this.device.get('name');
                var i;
                if (this.userGroupWarned === true) {
                    return true;
                }
                for (i = 0; i < devGroups.length; i++) {
                    if (dSS.util.isUserGroup(devGroups[i])) {
                        this.userGroupWarned = true;
                        box.collapse();
                        Ext.Msg.alert(dSS._("Warning"),
                            dSS.gt.strargs(
                                dSS._("Device %1 %2 is member in one or more " +
                                      "user groups. Changing device color " +
                                      "configuration will remove the device " +
                                      "from all user groups."),
                                [
                                    '&quot;' + name + '&quot;',
                                    '(' + this.device.get('DisplayID') + ')'
                                ]
                            )
                        );
                        return false;
                    }
                }
                return true;
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
                dSS.MessageBus.removeListener('inputtypechanged',
                                              this.updateVisibility, this);
            },
            this);

        this.on('submit', function() {
                if (this.disabled) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                var value = this.colorSelection.getValue();
                var dSUID = this.dSUID;
                var group = this.group;
                if (value == group) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                if (this.matrix.setumr200config === true) {
                    this.fireEvent('formsetdeviceconfig',
                        {
                            application: dSS.config.GroupV2Map[value]
                        });
                    return;
                }

                Ext.Ajax.request({
                    url: '/json/device/setJokerGroup',
                    disableCaching: true,
                    method: "GET",
                    params:
                    {
                        dsuid : this.dSUID,
                        groupID : value
                    },
                    scope: this,
                    success: function(result, request) {
                        try {
                            var json = Ext.util.JSON.decode(result.responseText);
                            if (!json.ok) {
                                Ext.Msg.alert(dSS._("Error"),
                                    dSS._("Could not set joker device group") +
                                    ': (' + json.message + ')');
                                this.fireEvent('formsubmitted');
                                return;
                            }
                            dSS.util.processDeviceActionResponse(json.result);
                        } catch (err) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not set joker device group") +
                                ': (' + err + ')');
                        }
                        this.fireEvent('formsubmitted');
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set joker device group"));
                        this.fireEvent('formsubmitted');
                    }
                });
        },
        this);

        dSS.MessageBus.on('inputtypechanged', this.updateVisibility, this);
    },
    updateVisibility: function(input) {
        if (input == dSS.config.InputTypeHack.SENSOR) {
            this.hide();
            this.disable();
        } else if (input == dSS.config.InputTypeHack.PUSHBUTTON) {
            this.show();
            this.enable();
        }
        this.fireEvent('updateshadow');
    }
});

Ext.reg('dssdevjokerconfigform', dSS.devprops.JokerConfigForm);

