Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: device
dSS.devprops.OperationLockIgnoreConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.titleLabel = new Ext.Panel(
        {
            border: false,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    height: 30,
                    text: dSS._("Ignore operation lock for weather alarms") + ":",
                    width: 400,
                    style: { marginTop: '5px' }
                }
            ]
        });

        var radioItems =
        [
            {
                boxLabel: dSS._("never ignore lock message (recommended)"),
                name: 'rb_operation_lock_' + config.dSUID,
                id: 'no_operation_lock',
                inputValue : 0
            },
            {
                boxLabel: dSS._("ignore operation lock message"),
                name: 'rb_operation_lock_' + config.dSUID,
                id: 'operation_lock',
                inputValue : 1
            }
        ];

        this.radio = new Ext.form.RadioGroup(
        {
            bubbleEvents: [ 'change' ],
            columns: 1,
            items: radioItems,
            height: 50
        });

        this.valueLoaded = this.device.get('ignoreOperationLock') ? 'operation_lock' : 'no_operation_lock';
        this.radio.setValue(this.valueLoaded, true);

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'formretry' ],
            padding: 5,
            height: 100,
            scope: this,
            border: false,
            items:
            [
                this.titleLabel,
                this.radio
            ]
        });

        dSS.devprops.OperationLockIgnoreConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
            this.on('change', function(rgroup, rbutton) {
                    if (rgroup.getValue().getId() == 'operation_lock') {
                        Ext.Msg.show({
                            title: dSS._("Ignore operatoin lock"),
                            msg: dSS._("By ignoring the operation lock, the " +
                                    "shade might be operated by the user " +
                                    "despite an active weather alarm. When " +
                                    "continuing, you acknowledge to take " +
                                    "full responsibility for any damage " +
                                    "caused due to the manual operation."),
                            buttons: Ext.Msg.YESNO,
                            icon: Ext.MessageBox.QUESTION,
                            scope: this,
                            animEl: 'elId',
                            fn: function(button, text) {
                                if (button == 'yes') {
	                                this.fireEvent('formchanged');
                                } else {
                                    rgroup.setValue(this.valueLoaded, true);
                                }
                            }
                        });
                    } else {
                        this.fireEvent('formchanged');
                    }
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            var value;
            // this happens if the value was not loaded correctly
            if (this.radio.getValue() != null) {
                value = this.radio.getValue().getId();
            }

            if (this.valueLoaded == value) {
                this.fireEvent('formsubmitted');
                return;
            }

            // do not submit values that have not changed
            dSS.util.genericRequest.defer(2000, this,
            [
                {
                    url: '/json/device/setIgnoreOperationLock',
                    params:
                    {
                        dsuid: this.dSUID,
                        lockStatus: (value == 'operation_lock') ? 1 : 0
                    },
                    failureMsg: dSS._("Could not save ignore operation lock configuration"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
            this.device.set('ignoreOperationLock', (value == 'operation_lock'));
            this.device.commit();
        },
        this);
    }
});

Ext.reg('operationlockignoreconfigform', dSS.devprops.OperationLockIgnoreConfigForm);

