Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        this.dSUID = this.device.get('dsuid');
        this.dialogType = config.type;
        this.group = config.group;
        this.matrix = config.matrix;
        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.loaders = 1;
        if (this.matrix.outmodeauto === true) {
            this.loaders = 2;
        }
        this.loadedCount = 0;

        this.loadedValue = undefined;

        this.registeredTips = [];

        this.magicNumber_rb2 = 662;

        var disableUMRCombinedModes = ((this.matrix.umroutmodeswitch2x === true) &&
                (dSS.util.getJokerGroup(this.device.get('groups')) == dSS.config.groups.TEMPCONTROL));

        var height;
        var radioItems = [];

        if (this.matrix.umroutmodeswitch2x === true) {
            height = 130;
        }

        if (this.matrix.outmodeenoceanvalve === true) {
            radioItems =
            [
                {
                    boxLabel: dSS._("pulse width modulation (PWM)"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: this.magicNumber_rb2.toString(),
                    inputValue: this.magicNumber_rb2
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue: dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.heatingoutmode === true) {
            radioItems =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue: dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("pulse width modulation (PWM)"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: this.magicNumber_rb2.toString(),
                    inputValue: this.magicNumber_rb2
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue: dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.extradimmer === true) {
            radioItems =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("dimmed 0-10V"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER_0_10V
                },
                {
                    boxLabel: dSS._("dimmed 1-10V"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER_1_10V
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.umroutmode === true) {
            this.sswitchedText = dSS._("Both relais are switched independently.");
            radioItems =
            [
                {
                    boxLabel: dSS._("single switched"),
                    ref: 'singleSwitchedRB',
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL
                }
            ];

            if (this.device.get('dSUIDIndex') < 3) {
                this.cswitchedText = dSS._("Both relais are switched toghether.");
                this.c2sswitchedText = dSS._("Both relais switch combined in two steps depending on the output value. Output > 33% - relais 1 is on. Output > 66% - relais 1 and relais 2 are on.");
                this.c3sswitchedText = dSS._("Both relais switch combined in three steps depending on the output value. Output > 25% - relais 1 is on. Output > 50% - relais 1 is off and relais 2 is on. Output > 75% - relais 1 and relais 2 are on.");
                this.rbc1 = new Ext.form.Radio({
                    boxLabel: dSS._("combined switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    disabled: disableUMRCombinedModes,
                    id: dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH
                });
                radioItems.push(this.rbc1);

                this.rbc2 = new Ext.form.Radio({
                    boxLabel: dSS._("combined two stage switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    disabled: disableUMRCombinedModes,
                    id: dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH
                });
                radioItems.push(this.rbc2);

                this.rbc3 = new Ext.form.Radio({
                    boxLabel: dSS._("combined three stage switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    disabled: disableUMRCombinedModes,
                    id: dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH
                });
                radioItems.push(this.rbc3);
            }

            this.disabledText = dSS._("Both relais outputs are disabled.");
            radioItems.push(
            {
                boxLabel: dSS._("disabled"),
                name: 'rb_outmode_' + config.dSUID,
                id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
            });
        } else if (this.matrix.outmodeswitch === true) {
            radioItems =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else if (this.matrix.outmodegeneric === true) {
            radioItems =
            [
                {
                    boxLabel: dSS._("active"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DEFAULT_ACTIVE.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DEFAULT_ACTIVE
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        } else {
            radioItems =
            [
                {
                    boxLabel: dSS._("switched"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                },
                {
                    boxLabel: dSS._("dimmed"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER
                },
                {
                    boxLabel: dSS._("disabled"),
                    name: 'rb_outmode_' + config.dSUID,
                    id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                    inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                }
            ];
        }

        this.radio = new Ext.form.RadioGroup(
        {
            bubbleEvents: [ 'change' ],
            fieldLabel: dSS._("Output mode"),
            columns: 1,
            items: radioItems,
            height: height
        });

        var items = [];
        var items2 = [];
        var layout = 'form';

        if (this.matrix.outmodeauto === true) {
            this.automodelabel = new Ext.form.Label({
                height: 20,
                flex: 4,
                style: { marginTop: '5px' }
            });
        }

        if (this.matrix.dimmodeconfig === true) {
            this.dimselection = new Ext.form.ComboBox({
                x: 0,
                y: 25,
                width: 150,
                typeAhead: false,
                triggerAction: 'all',
                lazyender: true,
                forceSelection: true,
                editable: false,
                disabled: true,
                mode: 'local',
                store: new Ext.data.ArrayStore({
                    id: 0,
                    fields:
                    [
                        'id',
                        'displayText'
                    ],
                    data:
                    [
                        [
                            dSS.config.deviceParams.OUTPUT_MODE_LEADING_EDGE_DIMMER,
                            dSS._("leading edge")
                        ],
                        [
                            dSS.config.deviceParams.OUTPUT_MODE_DIMMER,
                            dSS._("trailing edge")
                        ],
                        [
                            dSS.config.deviceParams.OUTPUT_MODE_AUTO,
                            dSS._("automatic")
                        ]

                        // #20874 - these modes may reappear later, temporarily
                        // commenting them out
                        /*
                        [
                            dSS.config.deviceParams.OUTPUT_MODE_RMS_DIMMER,
                            dSS._("root mean square")
                        ],
                        [
                            dSS.config.deviceParams.OUTPUT_MODE_PULSE_PACKAGE_CTL_DIMMER,
                            dSS._("pulse package control")
                        ]
                        */
                    ]
                }),
                valueField: 'id',
                displayField: 'displayText'

            });

            this.dimwarnlabel = new Ext.form.Label({
                flex: 3
            });

            this.iconRL = new Ext.BoxComponent({
                autoEl: { tag: 'img', src: '/images/RL.png' },
                x: 155,
                y: 25,
                hidden: true
            });

            this.iconRC = new Ext.BoxComponent({
                autoEl: { tag: 'img', src: '/images/RC.png' },
                x: 155,
                y: 25,
                hidden: true
            });

            this.iconRLC = new Ext.BoxComponent({
                autoEl: { tag: 'img', src: '/images/RLC.png' },
                x: 155,
                y: 25,
                hidden: true
            });

            this.testOutputButton = new Ext.Button({
                text: dSS._("Test output"),
                width: 180,
                scope: this,
                disabled: true,
                handler: this.testOutput.createDelegate(this)
            });
        }

        if ((this.matrix.outmodeauto === true) ||
            (this.matrix.dimmodeconfig === true)) {
            items.push(
            {
                xtype: 'panel',
                border: false,
                width: 215,
                layout: 'form',
                items:
                [
                    this.radio
                ]
            });

            var subitems = [];

            if (this.matrix.dimmodeconfig === true) {
                subitems.push(this.dimselection);
                subitems.push(this.iconRL);
                subitems.push(this.iconRC);
                subitems.push(this.iconRLC);
            }

            items.push(
            {
                xtype: 'panel',
                border: false,
                flex: 3,
                layout: 'absolute',
                height: 60,
                items: subitems
            });

            layout = 'hbox';
        } else {
            items = [ this.radio ];
        }

        var items3 = [];
        var lastLine =
        {
            xtype: 'panel',
            border: false,
            layout: 'hbox',
            height: 25,
            width: 408,
            items: []
        };

        if (this.matrix.outmodeauto === true) {
            lastLine.items.push(this.automodelabel);
        } else {
            lastLine.items.push({ xtype: 'box', flex: 3 });
        }

        if (this.matrix.dimmodeconfig === true) {
            items3.push(this.dimwarnlabel);
            lastLine.items.push(this.testOutputButton);
        }

        if (lastLine.items.length > 0) {
            items3.push(lastLine);
        }

        items2.splice(0, 0,
        {
            xtype: 'panel',
            border: false,
            layout: layout,
            items: items
        });

        if (items3.length > 0) {
            items2.push(
            {
                xtype: 'panel',
                border: false,
                layout: 'vbox',
                height: 100,
                items: items3
            });
        }

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged',
                            'cancelbuttonstate'],
            padding: 5,
            scope: this,
            border: false,
            layout: 'form',
            items: items2
        });

        dSS.devprops.OutputModeForm.superclass.constructor.apply(this, arguments);

        if (this.matrix.heatingoutmode === true) {
            dSS.MessageBus.addListener('heatinggroupformchanged',
                this.updateHeatingGroup, this);
        }

        if (this.matrix.outmodetempcontrol === true) {
            dSS.MessageBus.addListener('jokerconfigformchanged',
                this.updateJokerGroup, this);
        }

        this.on('loadcomplete', function() {
                this.on('change', function() {
                    this.fireEvent('formchanged');
                },
                this);
                if (this.sswitchedText) {
                    this.registerRBTip(this.sswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL.toString());
                }

                if (this.cswitchedText) {
                    this.registerRBTip(this.cswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH.toString());
                }

                if (this.c2sswitchedText) {
                    this.registerRBTip(this.c2sswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH.toString());
                }

                if (this.c3sswitchedText) {
                    this.registerRBTip(this.c3sswitchedText,
                        dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH.toString());
                }

                if (this.disabledText) {
                    this.registerRBTip(this.disabledText,
                        dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString());
                }
                if ((this.matrix.outmodeauto === true) ||
                   (this.matrix.dimmodeconfig === true)) {
                    this.radio.on('change', function(radio, item) {
                        if (this.matrix.dimmodeconfig === true) {
                            if (item.getRawValue() == dSS.config.deviceParams.OUTPUT_MODE_DIMMER) {
                                if (this.dimselection.getValue() == "") {
                                    this.dimselection.setValue(dSS.config.deviceParams.OUTPUT_MODE_AUTO);
                                }
                                this.processAdditionalElements(this.dimselection.getValue());
                                this.enableDimSelectionBlock(true);
                            } else {
                                this.enableDimSelectionBlock(false);
                                this.testOutputButton.disable();
                            }
                        }
                    },
                    this);
                }

                if (this.matrix.dimmodeconfig === true) {
                    this.dimselection.on('select', function(combo, record) {
                        this.processAdditionalElements(record.get('id'));
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader == dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    this.loadedCount++;
                    if (data.json.ok !== true) {
                        this.radio.markInvalid(data.json.message);
                    } else {
                        var val = data.json.result[this.dSUID][0].outputMode;
                        this.loadedValue = val;
        
                        if (this.matrix.dimmodeconfig === true) {
                            if (this.isDimMode(val)) {
                                this.dimselection.setValue(val);
                                this.enableDimSelectionBlock(true);
                                if (val == dSS.config.deviceParams.OUTPUT_MODE_AUTO) {
                                    this.testOutputButton.enable();
                                }
                                this.processAdditionalElements(this.dimselection.getValue());
                                val = dSS.config.deviceParams.OUTPUT_MODE_DIMMER;
                            } else {
                                this.enableDimSelectionBlock(false);
                            }
                        }

                        // the radio button has a value of 35, so we need to
                        // adapt when loading; the submit code will tune it
                        // back to 65 if we are still in the TC group
                        if ((this.matrix.outmodetempcontrol === true) &&
                            (val == dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED)) {
                                val = dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL;
                            }
                        this.setRadioGroup(val);
                    }
                }

                if ((this.matrix.outmodeauto === true) &&
                    (data.loader == dSS.devprops.Loaders.CURRENT_OUTPUT_AUTO_MODE)) {
                    this.loadedCount++;
                    var val;
                    if (data.json.ok === true) {
                        val = data.json.result.value;
                        this.loadedCurrentMode = val;
                    }
                    this.updateAutomodeLabel(val);

                }

                if (this.loadedCount >= this.loaders) {
                    this.el.unmask();
                    this.fireEvent('formloaded', true);
                }
            },
            this);

        this.on('beforedestroy', function() {
                var i;
                for (i = 0; i < this.registeredTips.length; i++) {
                    Ext.QuickTips.unregister(this.registeredTips[i]);   
                }
                this.closed = true;
                if (this.matrix.heatingoutmode === true) {
                    dSS.MessageBus.removeListener('heatinggroupformchanged',
                        this.updateGroup, this);
                }

                if (this.matrix.outmodetempcontrol === true) {
                    dSS.MessageBus.removeListener('jokerconfigformchanged',
                        this.updateJokerGroup, this);
                }
            },
            this);

        this.on('submit', function() {
            this.saveSettings('formsubmitted');
        }, this);
    },

    saveSettings: function(eventName) {
        var value;
        // this happens if the value was not loaded correctly
        if (this.radio.getValue() != null) {
            value = this.radio.getValue().getRawValue();
            value = this.convertValueFromMagic(value);
        }

        if ((this.matrix.outmodetempcontrol === true) &&
            (this.group == dSS.config.groups.TEMPCONTROL) &&
            (value == dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_POL)) {
            value = dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED;
        }

        if (this.matrix.dimmodeconfig === true) {
            if (value == dSS.config.deviceParams.OUTPUT_MODE_DIMMER) {
                var subvalue = this.dimselection.getValue();
                value = subvalue;
            }
        }

        if((this.matrix.heatingoutmode === true)  &&
            (this.group == dSS.config.groups.TEMPCONTROL) &&
            (value == dSS.config.deviceParams.OUTPUT_MODE_SWITCH)) {
            value = dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED;
        }
        // do not submit anything if value was not loaded and also has
        // not been changed by the user
        if ((value === undefined) || (value == this.loadedValue)) {
            this.fireEvent(eventName);
            return;
        }

        var dSUID = this.dSUID;

        Ext.Ajax.request({
            url: '/json/device/setOutputMode',
            method: 'GET',
            disableCaching: true,
            scope: this,
            params:
            {
                dsuid : dSUID,
                modeID : value
            },
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok !== true) {
                        throw "error";
                    }

                    var store = dSS.data.Loader.getDeviceStore();
                    var record = store.getById(dSUID);
                    if (record !== undefined) {
                        record.set('outputMode', parseInt(value, 10));
                        record.commit();
                    }

                    dSS.util.processDeviceActionResponse(jsonData.result);
                    this.fireEvent(eventName);
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not save output mode settings"));
                    this.fireEvent(eventName);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not save output mode settings"));
                this.fireEvent(eventName);
            }
        });
    },
    registerRBTip: function(text, rbid) {
        // the tooltips are ultimately a hack because radio button
        // tooltips are not supported by default
        var formel = Ext.get('x-form-el-' + rbid);
        if (formel) {
            var tipel = formel.first();
            if (tipel) {
                Ext.QuickTips.register({
                    text: text,
                    target: tipel.id,
                    trackMouse: false,
                    dismissDelay: 10000
                });
                this.registeredTips.push(tipel.id);
            }
        }
    },

    testOutput: function() {
        this.el.mask(dSS._("Testing output") + "...", 'x-mask-loading');
        this.fireEvent('cancelbuttonstate', false);
        this.fireEvent('formchanged');

        var thiz = this;

        var setval = function(scene, callback) {
            dSS.util.genericRequest(
                {
                    url: '/json/device/callScene',
                    params:
                    {
                        dsuid: thiz.dSUID,
                        sceneNumber: scene,
                        category: 'manual'
                    },
                    scope: thiz
                },
                callback.createDelegate(thiz)
            );
        };

        var callbackMode = function(success, data) {
            var val;
            if (success == true) {
                val = data.result.value;
                thiz.loadedCurrentMode = val;
            }

            thiz.updateAutomodeLabel(val);
            thiz.el.unmask();
        };

        var callbackMax = function(success, data) {
            dSS.util.genericRequest.defer(1000, thiz,
                [
                    {
                        url: '/json/device/getOutputValue',
                        params:
                        {
                            dsuid: thiz.dSUID,
                            type: 'currentOutputMode'
                        }
                    },
                    callbackMode.createDelegate(thiz)
                ]
            );

        };

        var callbackMin = function(success, data) {
            setval.defer(1000, thiz, [ dSS.config.scenes.MAX, callbackMax ]);
        };

        this.on('internalsubmit', function() {
            setval(dSS.config.scenes.MIN, callbackMin);
        }, this, { single: true });

        this.saveSettings('internalsubmit');
    },

    enableDimSelectionBlock: function(enable) {
        this.dimselection.setDisabled(!enable);
        this.iconRL.setDisabled(!enable);
        this.iconRC.setDisabled(!enable);
        this.iconRLC.setDisabled(!enable);
        if (!enable) {
          this.dimwarnlabel.setText("");
        }
    },

    processAdditionalElements: function(value) {
        this.iconRL.hide();
        this.iconRC.hide();
        this.iconRLC.hide();
        this.dimwarnlabel.setText("");
        this.testOutputButton.disable();

        var linktext = dSS._("More information");
        var url = '<span class="x-text-link"><a href="http://products.digitalstrom.com/' + this.device.get('GTIN') + '" target="_blank">' + linktext + '</a></span>';
        var warn = '<b>' + dSS._("Using a not permitted load could damage the dimmer or the load!") + ' - ' + url + '</b>';

        if (value == dSS.config.deviceParams.OUTPUT_MODE_LEADING_EDGE_DIMMER) {
            this.iconRL.show();
            this.dimwarnlabel.update(warn);
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_DIMMER) {
            this.iconRC.show();
            this.dimwarnlabel.update('<b>' +
                    dSS._("Using the selected output mode " +
                          "with inductive loads could damage the dimmer!") +
                    ' - ' + url + '</b>');
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_AUTO) {
            this.iconRLC.show();
            this.dimwarnlabel.update(warn);
            this.testOutputButton.enable();
        }
    },

    updateHeatingGroup: function(group) {
        if (this.matrix.heatingoutmode !== true) {
            return;
        }

        this.group = group;
    },

    tuneSelection: function(value) {
        switch (value)
        {
            case dSS.config.deviceParams.OUTPUT_MODE_BIPOLAR_SWITCH:
            case dSS.config.deviceParams.OUTPUT_MODE_TWO_STAGE_SWITCH:
            case dSS.config.deviceParams.OUTPUT_MODE_THREE_STAGE_SWITCH:
            case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_2_STEPS:
            case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_IN_3_STEPS:
            case dSS.config.deviceParams.OUTPUT_MODE_SWITCH_2_OUT_PARALLEL:
                this.radio.setValue(dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(), true);
                break;
            default:
                if ((value != null) && (value != undefined)) {
                    this.radio.setValue(value, true);
                }
                break;
        }
    },
    updateJokerGroup: function(group) {
        if (this.matrix.umroutmodeswitch2x !== true) {
            this.group = group;
            return;
        }

        var value;
        if (this.radio.getValue() != null) {
            value = this.radio.getValue().getRawValue();
            value = this.convertValueFromMagic(value);
        }

        if ((this.group == dSS.config.groups.TEMPCONTROL) &&
            (group != dSS.config.groups.TEMPCONTROL)) {
            this.rbc1.enable();
            this.rbc2.enable();
            this.rbc3.enable();
            if ((value != null) && (value != undefined)) {
                this.radio.setValue(value, true);
            }
        } else if ((this.group != dSS.config.groups.TEMPCONTROL) &&
                   (group == dSS.config.groups.TEMPCONTROL)) {
            this.tuneSelection(value);
            this.rbc1.disable();
            this.rbc2.disable();
            this.rbc3.disable();
        }
        this.group = group;
    },

    setRadioGroup: function(value) {
        if (this.matrix.heatingoutmode === true) {
            if ((value === dSS.config.deviceParams.OUTPUT_MODE_CONTROL_SWITCHED) ||
                (value === dSS.config.deviceParams.OUTPUT_MODE_SWITCH)) {
                value = dSS.config.deviceParams.OUTPUT_MODE_SWITCH;
            } else if (dSS.util.isHeatingPWMMode(value)) {
                value = this.magicNumber_rb2;
            }
        }

        if((this.matrix.outmodeenoceanvalve === true) &&
            (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED)) {
            value = this.magicNumber_rb2;
        } else if (this.matrix.outmodegeneric === true) {
            if (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED) {
                value = dSS.config.deviceParams.OUTPUT_MODE_DEFAULT_ACTIVE;
            }
        }

        if ((this.matrix.umroutmodeswitch2x === true) &&
            (this.group == dSS.config.groups.TEMPCONTROL)) {
            this.tuneSelection(value);
            this.rbc1.disable();
            this.rbc2.disable();
            this.rbc3.disable();
        } else {
            this.radio.setValue(value, true);
            if (this.matrix.umroutmodeswitch2x === true) {
                this.rbc1.enable();
                this.rbc2.enable();
                this.rbc3.enable();
            }
        }
    },

    convertValueFromMagic: function(value) {
        value = parseInt(value);

        if((this.matrix.heatingoutmode === true) ||
           (this.matrix.outmodeenoceanvalve === true)) {
            if (value === this.magicNumber_rb2) {
                if (this.group == dSS.config.groups.TEMPCONTROL) {
                    return dSS.config.deviceParams.OUTPUT_MODE_CONTROL_PWM;
                } else {
                    return dSS.config.deviceParams.OUTPUT_MODE_PWM;
                }
            }
        }
        return value; // 0 = disabled is same for both groups
    },
    updateAutomodeLabel: function(value) {
        if (this.matrix.outmodeauto !== true) {
            return;
        }

        var mode = "";

        if (value == dSS.config.deviceParams.OUTPUT_MODE_LEADING_EDGE_DIMMER) {
            mode = dSS._("leading edge");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_DIMMER ) {
            mode = dSS._("trailing edge");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_RMS_DIMMER ) {
            mode = dSS._("RMS");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_PULSE_PACKAGE_CTL_DIMMER ) {
            mode = dSS._("pulse package control");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_DISABLED) {
            mode = dSS._("disabled");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_SWITCH) {
            mode = dSS._("switched");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_AUTO) {
            mode = dSS._("not detected");
        } else if (value == dSS.config.deviceParams.OUTPUT_MODE_ERROR) {
            mode = dSS._("error");
        } else {
            // #21094: notify user about bogus modes
            mode = dSS._("invalid");
        }

        if (mode.length > 0) {
            this.automodelabel.setText(dSS._("Current mode") + ': ' + mode);
        } else {
            this.automodelabel.setText("");
        }
    },
    isDimMode: function(mode) {
        switch (mode)
        {
            case dSS.config.deviceParams.OUTPUT_MODE_RMS_DIMMER:
            case dSS.config.deviceParams.OUTPUT_MODE_LEADING_EDGE_DIMMER:
            case dSS.config.deviceParams.OUTPUT_MODE_DIMMER:
            case dSS.config.deviceParams.OUTPUT_MODE_PULSE_PACKAGE_CTL_DIMMER:
            case dSS.config.deviceParams.OUTPUT_MODE_AUTO:
                return true;
            default:
                return false;
        }
    }
});

Ext.reg('dssdevoutputmodeform', dSS.devprops.OutputModeForm);

