Ext.namespace("dSS", "dSS.config", "dSS.devprops");

dSS.devprops.SinglePresence = Ext.extend(Ext.Window,
{
    constructor: function(config)
    {
        if (config.width === undefined)
        {
            config.width = 480;
        }

        this.timeout = config.timeout;
        if (this.timeout === undefined) {
            this.timeout = 12;
        }

        this.maskElement = config.maskElement;

        this.dsuid = config.dsuid;
        this.dsmdsuid = config.dsmdsuid;
        this.name = config.name;
        this.wasPresent = config.wasPresent;

        this.closeButton =
        {
            xtype: 'button',
            text: dSS._("Close"),
            scope: this,
            handler: function()
            {
                this.closed = true;
                this.close();

                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
            }
        };

        var infopanel = dSS.util.CreateDefaultDeviceInfoPanel(config.name,
                                                               config.displayid,
                                                                 true);
        Ext.apply(this,
        {
            title: dSS._("Update Device Status"),
            bodyBorder: false,
            padding: 5,
            resizable: false,
            items:
            [
                {
                    xtype: 'panel',
                    ref: 'mainpanel',
                    padding: 5,
                    border: false,
                    bodyBorder: false,
                    frame: false,
                    bodyCfg:
                    {
                        cls: 'x-window-body'
                    },
                    items:
                    [
                        infopanel,
                        {
                            xtype: 'panel',
                            padding: 5,
                            ref: 'datapanel',
                            cls: 'x-text-align-left x-text-nowrap',
                            html: dSS._("Device status") + ':&nbsp;...'
                        }
                    ]
                }
            ],
            buttonAlign: 'right',
            iconCls: 'loading-indicator-small',
            buttons:
            [
                this.closeButton
            ]
        });

        dSS.devprops.SinglePresence.superclass.constructor.apply(this, arguments);

        this.on('afterrender',
            function()
            {
                if (this.maskElement !== undefined)
                {
                    this.maskElement.mask();
                }

                this.tID = Ext.Ajax.request({
                        url: '/json/device/testPresence',
                        disableCaching: true,
                        method: "GET",
                        params: {
                            lang: dSS.config.language,
                            dsuid: this.dsuid
                        },
                        success: function(result, request) {
                            try {
                                var jsonData = Ext.util.JSON.decode(result.responseText);
                                if (jsonData && jsonData.ok && jsonData.result) {
                                    this.callback(true, jsonData.result.active);
                                } else {
                                    throw "Invalid reply";
                                }
                            }
                            catch(err) {
                                this.callback(false);
                            }
                        }.createDelegate(this),
                        failure: function(result, request) {
                          this.callback(false);
                        }.createDelegate(this)
                    });
            },
            this);

        this.on('beforedestroy',
            function()
            {
                if (this.tID !== undefined)
                {
                    Ext.Ajax.abort(this.tID);
                }

                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
            },
            this);

        this.callback = function(success, isPresent)
        {
            this.tID = undefined;

            if (this.closed === true) {
                return;
            }

            if (!success) {
                this.close();
                Ext.Msg.alert(dSS._("Error"),dSS._("Device presence test failed."));
                return;
            }

            this.setIconClass('deviceTestPresenceIcon');

            this.mainpanel.datapanel.update(dSS._("Device status") + ':&nbsp;' +
                                (isPresent ? dSS._("Active") : dSS._("Inactive")));

            if (isPresent != this.wasPresent) {
                dSS.MessageBus.fireEvent('reloadapartment');
                dSS.config.firstLoad = false;
            }
        }
    }
});
