Ext.namespace("dSS", "dSS.config", "dSS.devprops");

dSS.devprops.VentilationConfig = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        var list =
        [
            [
                dSS.config.groups.VENTILATION,
                dSS._(dSS.config.groupNames[dSS.config.groups.VENTILATION])
            ],
            [
                dSS.config.groups.RECIRCULATION,
                dSS._(dSS.config.groupNames[dSS.config.groups.RECIRCULATION])
            ],
            [
                dSS.config.groups.APT_VENTILATION,
                dSS._(dSS.config.groupNames[dSS.config.groups.APT_VENTILATION])
            ]
        ];


        this.dSUID = config.dSUID;
        this.present = config.present;

        this.loadedGroup = config.group;

        this.addEvents('formsubmitted', 'formloaded', 'sharedevent');

        this.ventSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            disabled: (this.present == false),
            width: 165,
            mode: 'local',
            fieldLabel: dSS._("Ventilation"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText',
            valueNotFoundText: " ",
            value: this.loadedGroup
        });

        Ext.apply(this, {
            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'formchanged', 'formdelayothers'
            ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'box',
                            height: 5
                        },
                        this.ventSelection
                    ]
                }
            ]
        });

        dSS.devprops.VentilationConfig.superclass.constructor.apply(this, arguments);

        this.on('afterrender', function() {
                this.ventSelection.on('select', function() {
                    this.fireEvent('formchanged');
                    this.fireEvent('formdelayothers', this.getId());
                },
                this);

                this.fireEvent('formloaded', true);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('sharedevent', function(name, data) {
                if (name != "ftwtempcontrolventilationselect") {
                    return;
                }

                if (data == "temperaturecontrolventilation") {
                    this.ventSelection.enable();
                } else if (data == "temperaturecontrol") {
                    this.ventSelection.disable();
                }
            },
            this);

        this.on('submit', function() {
                if (this.ventSelection.disabled) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                var value = this.ventSelection.getValue();
                var dSUID = this.dSUID;

                if ((value == undefined) || (value === this.loadedGroup)) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                dSS.util.genericRequest(
				{
                    url: '/json/device/setJokerGroup',
                    scope: this,
                    params:
                    {
                        dsuid: dSUID,
                        groupID: value
                    },
                    failureMsg: dSS._("Could not configure ventilation"),
					scope: this,
                    evt: 'formsubmitted',
                },
                function(success, json) {
                    if (success === true) {
                        dSS.util.processDeviceActionResponse(json.result);
                    }
                });
            },
            this);
    }
});

Ext.reg('dssdevventilationconfig', dSS.devprops.VentilationConfig);

