Ext.namespace("dSS", "dSS.config", "dSS.devprops");
dSS.devprops.VisibilityMatrix = {
    'GE' :  { },
    'GN' :  { },
    'RT' :  { },
    'GR' :  { },
    'BL' :  { },
    'TK' :  { },
    'SW' :  { }
};

dSS.devprops.ErrorMatrix = {};

dSS.devprops.GetVisibleProperties = function(record) {
    var none = dSS.devprops.ErrorMatrix;

    if (!record) {
        throw none;
    }

    var devInfo = dSS.util.parseProductID(record.get('productID'));
    var devType = devInfo[0];
    var devNumber = devInfo[1];
    var fInfo = dSS.util.parseFunctionID(record.get('functionID'));
    var devClass = fInfo[0];
    var modelUID = record.get('VdcModelUID');

    var dc = dSS.devprops.VisibilityMatrix[dSS.config.deviceColors[devClass]];
    if (dc === undefined) {
        throw none;
    }

    var ret;

    if (String(modelUID).length > 0) {
        ret = dc[modelUID];
        if (ret !== undefined) {
            return ret;
        } else {
            throw none;
        }
    }

    var key = dSS.config.deviceTypes[devType] + ':' + devNumber;
    ret = dc[key];
    if (ret === undefined) {
        // try wildcard - devType NNx
        key = dSS.config.deviceTypes[devType] + ':' +
              String(devNumber).charAt(0) + String(devNumber).charAt(1);
        ret = dc[key];
        if (ret === undefined) {
            // try wildcard - devType Nxx
            key = dSS.config.deviceTypes[devType] + ':' +
                  String(devNumber).charAt(0);
            ret = dc[key];
        }
    }

    if (ret === undefined) {
        throw none;
    }

    var copied = false;
    // apply overrides
    if (devType === dSS.config.deviceTypeNum.UMR) {
        var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));

        // only dSUID 1. and 2. have input mode settings
        if (record.get('dSUIDIndex') > 1) {
            copy.optypeconfig = false;
            copy.pushbutton = false;
            copy.pushbadvanced = false;
            copy.twowayconfig = false;
            copy.akmdelay = false;
            copy.akmsensor = false;
            copy.akminput = false;

            // hack to allow to configure the temperature control group
            // on the new UMR in the joker configuration dialog, for
            // firmware versions 3.8.3 and later

            if (record.get('productRevision') >= 0x383) {
                copy.jokertempcontrol = true;

                if (record.get('dSUIDIndex') === 2) {
                    copy.umroutmodeswitch2x = true;
                }

                if (dSS.util.getJokerGroup(record.get('groups')) ==
                        dSS.config.groups.TEMPCONTROL) {
                    copy.heatingprops = true;
                }
                copy.valvetype = true;
            }
        }
        if (record.get('dSUIDIndex') < 2) {
            copy.blinkconfig = false;
            copy.umroutmode = false;
            copy.outvalue8 = false;
            copy.impulseconfig = false;
            copy.outconfigswitch = false;
            copy.enabletypehack = true;
        }

        ret = copy;
        copied = true;
    } else if (devType === dSS.config.deviceTypeNum.UMV) {
        // only dSUID 1. has umvrelay and extradimmer settings
        var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));
        if (record.get('dSUIDIndex') > 0) {
            copy.umvrelay = false;
        }
        ret = copy;
        copied = true;
    } else if ((devType === dSS.config.deviceTypeNum.SK) &&
               (devClass === dSS.config.deviceClass.BL) &&
               (devNumber === 204)) {
        // only the first dSUID has the full range of FTW settings
        if (record.get('dSUIDIndex') > 0) {
            var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));
            copy.ftwtempcontrolventilationselect = false;
            copy.ftwdisplaysettings = false;
            copy.ftwbacklighttimeout = false;
            copy.temperatureoffset = false;
            ret = copy;
            copied = true;
        }
    } else if (devType === dSS.config.deviceTypeNum.AKM) {
        var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));
        copy.enabletypehack = true;
        copy.inputtypeonlysensor = true;
        ret = copy;
        copied = true;
    } else if ((devType === dSS.config.deviceTypeNum.TKM) && (devNumber === 300)) {
        var copy = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));
        copy.enabletypehack = true;
        copy.inputtypeonlysensor = false;
        ret = copy;
        copied = true;
    }

    var overrideMF = record.get('modelFeatures');
    if (overrideMF !== undefined) {
        var overriden = ret;
        if (copied != true) {
            overriden = Ext.util.JSON.decode(Ext.util.JSON.encode(ret));
        }

        Ext.apply(overriden, overrideMF);
        ret = overriden;
    }

    return ret;
};
